"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _g = require("@antv/g6");
var _interface = require("./interface");
var _behavior = _interopRequireDefault(require("./behavior"));
var _shape = _interopRequireDefault(require("./shape"));
var _dataDeal = require("./util/dataDeal");
var _toolbar = _interopRequireDefault(require("./plugins/toolbar"));
var _StencilPlugin = _interopRequireDefault(require("./plugins/StencilPlugin"));
var _graph = require("./util/graph");
var _excluded = ["canSendMsg", "canSendWeChat", "canAuto", "abnormalProc", "expression", "remark"];
(0, _behavior["default"])();
(0, _shape["default"])();
var FlowDesigner = /*#__PURE__*/function () {
  function FlowDesigner(options) {
    var _this = this;
    (0, _classCallCheck2["default"])(this, FlowDesigner);
    this.graph = void 0;
    this.container = void 0;
    this.options = void 0;
    this.events = void 0;
    this.resizeTimer = void 0;
    this.selectedItemId = void 0;
    this.wheelEnable = void 0;
    this.flowTypes = void 0;
    this.usedFlowTypes = [];
    this.defaultData = void 0;
    this.setDynamicData = void 0;
    this.__onWheel = function (e) {
      var zoom = _this.graph.getZoom();
      if (e.deltaY > 0) {
        _this.graph.zoomTo(zoom - 0.1);
      } else {
        _this.graph.zoomTo(zoom + 0.1);
      }
    };
    this.__resizeFunc = function () {
      if (_this.resizeTimer) {
        clearTimeout(_this.resizeTimer);
      }
      _this.resizeTimer = setTimeout(function () {
        _this.graph.changeSize(_this.container.offsetWidth, _this.container.offsetHeight);
      }, 300);
    };
    this.container = options.container;
    this.options = options;
    this.events = options.events;
    this.setDynamicData = options.setDynamicData;
    this.graph = this.init(options);
    this.flowTypes = options.flowTypes || [];
    this.selectedItemId = null;
    this.wheelEnable = typeof options.wheelEnable === "undefined" ? true : options.wheelEnable;
    this.initEvent();
    this.defaultData = JSON.parse(JSON.stringify(options.defaultData || _dataDeal.defaultData));
  }
  (0, _createClass2["default"])(FlowDesigner, [{
    key: "init",
    value: function init(options) {
      var plugins = [];
      if (options.mode === 'edit') {
        var toolbar = new _toolbar["default"]({
          container: options.toolbar
        });
        var addItemPanel = new _StencilPlugin["default"]({
          container: options.itemWrapper,
          selector: options.itemSelector
        });
        plugins = [new _g.Grid({
          follow: false
        }), new _g.SnapLine({
          itemAlignType: 'center'
        }), toolbar, addItemPanel];
      }
      if (options.mode === 'view') {
        var tooltip = new _g.Tooltip({
          offsetX: 10,
          offsetY: 10,
          className: 'g6-tooltip',
          // the types of items that allow the tooltip show up
          // 允许出现 tooltip 的 item 类型
          itemTypes: ['node'],
          // custom the tooltip's content
          // 自定义 tooltip 内容
          getContent: function getContent(e) {
            var outDiv = document.createElement('div');
            outDiv.style.width = 'fit-content';
            outDiv.innerHTML = options.tooltip.formatText(e.item.getModel());
            return outDiv;
          },
          shouldBegin: options.tooltip.shouldBegin
        });
        plugins.push(tooltip);
      }
      var graph = new _g.Graph({
        plugins: plugins,
        container: options.container,
        width: options.width || options.container.offsetWidth,
        height: options.height || options.container.offsetHeight,
        modes: {
          "default": ['dragCanvas', 'clickSelected'],
          view: ['dragCanvas', 'clickSelected'],
          edit: ['brushSelect', 'dragCanvas', {
            type: 'drag-node',
            shouldBegin: function shouldBegin(e) {
              // 拖动锚点，则不响应当前行为
              return !/anchor|resize-point/.test(e.name);
            }
          }, 'clickSelected', 'dragEdge', 'dragAddEdge', 'dragAddNode', 'dragResize', 'hoverItem', 'keyboard']
        },
        defaultEdge: {
          type: 'flow'
        },
        defaultNode: {
          labelCfg: {
            style: {
              fontSize: 14
            }
          }
        },
        enabledStack: true,
        autoPaint: true,
        maxStep: 20
      });
      graph.setMode(options.mode);

      // 让画布可以响应拖动事件
      graph.get('canvas').set('draggable', true);
      graph.set('shouldConnect', _graph.shouldConnect);
      graph.set('beforeAddNode', _graph.beforeAddNode);
      graph.set('beforeAddEdge', _graph.beforeAddEdge);
      return graph;
    }
  }, {
    key: "initEvent",
    value: function initEvent() {
      var _this2 = this;
      var _this$events = this.events,
        onSelectedItem = _this$events.onSelectedItem,
        onBlurItem = _this$events.onBlurItem,
        onUpdateToolbar = _this$events.onUpdateToolbar,
        onItemClick = _this$events.onItemClick;
      this.graph.on('afterItemSelected', function (itemId) {
        if (itemId) {
          var item = _this2.graph.findById(itemId);
          if (item) {
            _this2.selectedItemId = itemId;
            onSelectedItem && onSelectedItem(itemId, item.getModel());
            if (item.getType() === 'node') {
              var edges = item.getOutEdges();
              edges.forEach(function (edge) {
                edge.toFront();
                _this2.graph.setItemState(edge, 'selected', true);
              });
            }
          } else {
            _this2.selectedItemId = null;
            onBlurItem && onBlurItem();
          }
        } else {
          _this2.selectedItemId = null;
          onBlurItem && onBlurItem();
        }
      });
      this.graph.on('updateToolbar', function (commands) {
        onUpdateToolbar && onUpdateToolbar(commands);
      });
      this.graph.on('itemClicked', function (itemId) {
        onItemClick && onItemClick(itemId, _this2.graph.findById(itemId));
      });
      this.wheelEnable && this.container.addEventListener('wheel', this.__onWheel);
      window.addEventListener('resize', this.__resizeFunc);
    }
  }, {
    key: "clearSelected",
    value: function clearSelected() {
      var _this3 = this;
      var mode = this.graph.getCurrentMode();
      var state = mode === 'edit' ? 'selected' : 'selected-view';
      var selected = this.graph.findAllByState('node', state);
      selected.forEach(function (node) {
        _this3.graph.setItemState(node, state, false);
      });
      selected = this.graph.findAllByState('edge', state);
      selected.forEach(function (edge) {
        _this3.graph.setItemState(edge, state, false);
      });
    }
  }, {
    key: "updateEdge",
    value: function updateEdge(item, value) {
      var _this4 = this;
      // 找到相关的边更新
      var edges = this.graph.getEdges();
      var targetEdges = edges.filter(function (edge) {
        return edge.getModel().source === _this4.selectedItemId || edge.getModel().target === _this4.selectedItemId;
      });
      targetEdges.forEach(function (edge) {
        var edgeModel = edge.getModel();
        console.log(edgeModel);
        var updateEdgeKey = edgeModel.source === _this4.selectedItemId ? 'sourceTaskDefKey' : 'tarTaskDefKey';
        _this4.graph.updateItem(edge, (0, _defineProperty2["default"])({}, updateEdgeKey, value));
      });
    }
  }, {
    key: "update",
    value: function update(key, cfg) {
      var item = this.graph.findById(this.selectedItemId);
      if (!item) return;

      // 更新label
      if (key === 'defName') {
        cfg.label = cfg.defName;
        cfg.labelCfg = {
          style: {
            fontSize: 14
          }
        };
      }
      this.graph.update(item, (0, _extends2["default"])({}, cfg));
      if (item.get('type') === 'edge') {
        var target = item.get('target');
        if (target.getModel().clazz === _interface.WorkFlowNodeEnum.SERVICE) {
          var outEdge = target.getOutEdges()[0];
          if (outEdge) {
            this.graph.update(outEdge, (0, _extends2["default"])({}, cfg));
          }
        }
      } else {
        // 更新节点相关的连线
        if (key === 'defKey' && item.get('type') === 'node') {
          this.updateEdge(item, cfg.defKey);
        }
      }
    }
  }, {
    key: "initData",
    value: function initData(data, dynamicTasks, options) {
      console.log('dynamicTasks: ', dynamicTasks);
      if (options && options.flowTypes) {
        this.flowTypes = options.flowTypes;
      }
      if (!data) {
        this.graph.data(this.defaultData);
      } else {
        this.graph.data((0, _dataDeal.dealFlowData)(data, this.flowTypes, this.usedFlowTypes, dynamicTasks, this.setDynamicData));
      }
      this.graph.render();
      this.graph.fitView(5);
    }
  }, {
    key: "save",
    value: function save() {
      var needValid = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
      var _ref = this.graph.save(),
        nodes = _ref.nodes,
        edges = _ref.edges;
      var tasks = {};
      var flows = {};
      var errorNode = null;
      if (needValid && !this.validate({
        nodes: nodes,
        edges: edges
      })) return;
      for (var i = 0, len = nodes.length; i < len; i++) {
        var _node = nodes[i];
        if (tasks[_node.defKey]) {
          errorNode = _node;
          break;
        }
        var mulInstKey = _node.mulInstKey || '';
        if (!mulInstKey) {
          if ([_interface.WorkFlowNodeEnum.PARALLEL_SIGN, _interface.WorkFlowNodeEnum.SERIAL_MULTI_SIGN].includes(_node.defCategory) || _node.isProcFirst && _node.parentGroupId) {
            mulInstKey = "mulInstKey_".concat(_node.groupId, "_").concat(_node.timeStmp);
          }
        }
        console.log('mulInstKey: ', mulInstKey);
        tasks[_node.defKey] = (0, _extends2["default"])({
          haveOpinionText: false,
          necOpinionText: false,
          canEditPage: false,
          haveAddSign: false,
          haveDelegate: false,
          passRead: false,
          haveCounterSign: false,
          canReadAtt: false,
          canDownloadAtt: false,
          canUploadAtt: false,
          canUpdateMyAtt: false,
          canUpdateOtherAtt: false,
          canDeleteMyAtt: false,
          canDeleteOtherAtt: false,
          canAppEdit: false,
          isProcFirst: false,
          taskTimeout: '',
          serviceTaskValue: '',
          compCondition: '',
          userChooseType: '',
          userChooseClass: '',
          userChooseMethod: '',
          showHideFields: '',
          editFields: ''
        }, _node, {
          mulInstKey: mulInstKey,
          nodeKey: _node.id,
          nodeIdIndex: i
        });
      }
      if (errorNode) {
        this.focusElement(errorNode, {
          translate: 'DUPLICATE_TASK_DEF_KEY',
          message: "\u4EFB\u52A1Key\u91CD\u590D: ".concat(errorNode.defKey, "\uFF0C\u8BF7\u91CD\u65B0\u586B\u5199")
        });
        return null;
      }
      edges.forEach(function (edge, lineIdIndex) {
        if (tasks[edge.sourceTaskDefKey].defCategory !== 'SERVICE') {
          flows[edge.id] = (0, _extends2["default"])({
            flowType: null,
            canSendMsg: false,
            canSendWeChat: false,
            canSendEMail: false,
            canAuto: false,
            abnormalProc: false,
            specialSubproc: false,
            withoutViewSubmit: false,
            expression: '',
            remark: ''
          }, edge, {
            sourceTaskNodeKey: edge.source,
            tarTaskNodeKey: edge.target,
            autoTime: edge.autoTime * 60 * 60 * 1000 || null,
            lineIdIndex: lineIdIndex
          });
        } else {
          // 服务节点出线和入线配置相同
          var inServiceEdge = edges.find(function (item) {
            return item.tarTaskDefKey === edge.sourceTaskDefKey;
          });
          if (!inServiceEdge) return;
          var canSendMsg = edge.canSendMsg,
            canSendWeChat = edge.canSendWeChat,
            canAuto = edge.canAuto,
            abnormalProc = edge.abnormalProc,
            expression = edge.expression,
            remark = edge.remark,
            others = (0, _objectWithoutProperties2["default"])(edge, _excluded);
          flows[edge.id] = (0, _extends2["default"])({
            canSendMsg: false,
            canSendWeChat: false,
            canSendEMail: false,
            canAuto: false,
            abnormalProc: false,
            specialSubproc: false,
            withoutViewSubmit: false,
            expression: '',
            remark: ''
          }, inServiceEdge, others, {
            sourceTaskNodeKey: edge.source,
            tarTaskNodeKey: edge.target,
            autoTime: inServiceEdge.autoTime * 60 * 60 * 1000 || '',
            lineIdIndex: lineIdIndex
          });
        }
      });
      return {
        tasks: tasks,
        flows: flows
      };
    }
  }, {
    key: "getData",
    value: function getData() {
      return this.save(false);
    }
  }, {
    key: "validate",
    value: function validate(data) {
      var _this5 = this;
      if (data.nodes && data.nodes.length === 0) {
        this.focusElement(null, {
          translate: 'PLEASE_ADD_TASK_NODE',
          message: "\u8BF7\u6DFB\u52A0\u6D41\u7A0B\u8282\u70B9"
        });
        return false;
      }
      var info = {
        translate: '',
        message: ''
      };
      var item = null;
      // 校验节点
      var nodePass = data.nodes.every(function (node) {
        if (!node.defKey) {
          info.translate = 'TASK_KEY_TIPS';
          info.message = "\u8BF7\u586B\u5199\u4EFB\u52A1Key";
          item = node;
          return false;
        }
        if (!node.defName) {
          info.translate = 'TASK_NAME_TIPS';
          info.message = "\u8BF7\u586B\u5199\u4EFB\u52A1\u540D\u79F0";
          item = node;
          return false;
        }
        if (/SERVICE/.test(node.clazz) && !node.serviceTaskValue) {
          info.translate = 'TASK_CODE_TIPS';
          info.message = "\u8BF7\u9009\u62E9\u4EFB\u52A1\u4EE3\u7801";
          item = node;
          return false;
        }
        var edges = _this5.graph.findById(node.id).getEdges();
        if (edges.length === 0) {
          info.translate = 'ISOLATED_TASK_NODE';
          info.message = "\u5B58\u5728\u5B64\u7ACB\u4EFB\u52A1\u8282\u70B9: ".concat(node.defName, "\uFF0C\u8BF7\u8FDE\u7EBF\u6216\u5220\u9664");
          item = node;
          return false;
        }
        if (!/START|END/.test(node.clazz)) {
          // 去除 开始 和 结束，其他节点都必须有出线和入线
          var inEdges = edges.filter(function (edge) {
            return edge.getTarget().get('id') === node.id;
          });
          var outEdges = edges.filter(function (edge) {
            return edge.getSource().get('id') === node.id;
          });
          // console.log(node, inEdges, outEdges);
          if (inEdges.length === 0) {
            info.translate = 'TASK_NOT_IN_EDGES';
            info.message = "\u4EFB\u52A1\u8282\u70B9: ".concat(node.defName, "\uFF0C\u6CA1\u6709\u5165\u7EBF\uFF0C\u8BF7\u8FDE\u7EBF");
            item = node;
            return false;
          }
          if (outEdges.length === 0) {
            info.translate = 'TASK_NOT_OUT_EDGES';
            info.message = "\u4EFB\u52A1\u8282\u70B9: ".concat(node.defName, "\uFF0C\u6CA1\u6709\u51FA\u7EBF\uFF0C\u8BF7\u8FDE\u7EBF");
            item = node;
            return false;
          }
        }

        // 子流程结束必须连入主流程
        if (node.parentGroupId && node.clazz === 'END') {
          var _outEdges = edges.filter(function (edge) {
            return edge.getSource().get('id') === node.id;
          });
          if (_outEdges.length === 0) {
            info.translate = 'SUB_PROC_END_NOT_OUTING';
            info.message = "\u5B50\u6D41\u7A0B\u7ED3\u675F\u8282\u70B9\uFF0C\u6CA1\u6709\u8FDE\u5165\u4E3B\u6D41\u7A0B\uFF0C\u8BF7\u8FDE\u7EBF";
            item = node;
            return false;
          }
        }
        return true;
      });
      if (!nodePass) {
        this.focusElement(item, info);
        return false;
      }

      // 校验连线
      var edgePass = data.edges.every(function (edge) {
        if (!edge.flowName) {
          info.translate = 'FLOW_NAME_INPUT_TIPS';
          info.message = "\u8BF7\u586B\u5199\u6D41\u5411\u540D\u79F0";
          item = edge;
          return false;
        }
        // 需要判断 流向目标的节点类型
        var sourceNode = _this5.graph.findById(edge.source).getModel();
        if (!/START/.test(sourceNode.clazz)) {
          if (!edge.flowType) {
            info.translate = 'FLOW_TYPE_INPUT_TIPS';
            info.message = "\u8BF7\u9009\u62E9\u6D41\u5411\u7C7B\u578B";
            item = edge;
            return false;
          }
        }
        var pattern = /^\${[\s\S]*}$/;
        if (edge.expression && !pattern.test(edge.expression)) {
          info.translate = 'JUDGE_CON_TIPS';
          info.message = "\u5224\u65AD\u6761\u4EF6\u7684\u6B63\u786E\u683C\u5F0F\u4E3A: " + '${key1 >= value1 && key2 < value2}';
          item = edge;
          return false;
        }
        return true;
      });
      if (!edgePass) {
        this.focusElement(item, info);
        return false;
      }
      return true;
    }
  }, {
    key: "focusElement",
    value: function focusElement(item, info) {
      if (this.selectedItemId) {
        this.graph.clearItemStates(this.selectedItemId, ['selected', 'hover']);
      }
      if (item) {
        this.selectedItemId = item.id;
        this.graph.setItemState(item.id, 'selected', true);
        this.graph.set('selectedItems', [item.id]);
      }
      this.events.onValidateError && this.events.onValidateError(item, info);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.wheelEnable && this.container.removeEventListener('wheel', this.__onWheel);
      window.removeEventListener('resize', this.__resizeFunc);
      this.graph.destroy();
    }
  }]);
  return FlowDesigner;
}();
exports["default"] = FlowDesigner;