"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = brushSelect;
var _g = require("@antv/g6");
var _graph = require("../util/graph");
// 范围选中节点

function brushSelect() {
  console.log('brushSelect');
  (0, _g.registerBehavior)('brushSelect', {
    getDefaultCfg: function getDefaultCfg() {
      return {
        brushStyle: {
          fill: '#9cc7ef',
          fillOpacity: 0.4,
          stroke: '#9cc7ef',
          lineWidth: 1,
          lineDash: [4, 3]
        },
        selectedNodes: []
      };
    },
    getEvents: function getEvents() {
      return {
        dragstart: 'onMouseDown',
        drag: 'onMouseMove',
        dragend: 'onMouseUp',
        'canvas:click': 'clearStates'
      };
    },
    onMouseDown: function onMouseDown(e) {
      // 不应该框选: 按在元素节点上面拖动时候 或者 当前正在拖动画布
      if (this.graph.get('canvasDragging')) return;
      if (e.item) return;
      this.graph.get('canvas').get('el').style.cursor = 'crosshair';
      var event = e.originalEvent;
      if (event && e.name !== 'touchstart' && event.button !== 0) {
        return;
      }
      this.clearStates();
      if (!this.brush) {
        this.brush = this.createBrush();
      }
      this.originPoint = {
        x: e.canvasX,
        y: e.canvasY
      };
      this.brush.attr({
        width: 0,
        height: 0
      });
      this.brush.show();
      this.dragging = true;
    },
    onMouseMove: function onMouseMove(e) {
      if (this.graph.get('canvasDragging') || !this.dragging || !this.originPoint || !this.brush) return;
      this.graph.set('brushSelecting', true);
      this.updateBrush(e);
      this.graph.get('canvas').get('el').style.cursor = 'crosshair';
    },
    onMouseUp: function onMouseUp(e) {
      if (!this.brush || !this.dragging || this.graph.get('canvasDragging')) return;
      this.brush.remove(true);
      this.brush = null;
      this.getSelectedNodes(e);
      this.dragging = false;
      this.graph.get('canvas').get('el').style.cursor = 'default';
    },
    updateBrush: function updateBrush(e) {
      var _this$brush;
      var originPoint = this.originPoint;
      (_this$brush = this.brush) === null || _this$brush === void 0 ? void 0 : _this$brush.attr({
        width: Math.abs(e.canvasX - originPoint.x),
        height: Math.abs(e.canvasY - originPoint.y),
        x: Math.min(e.canvasX, originPoint.x),
        y: Math.min(e.canvasY, originPoint.y)
      });
    },
    getSelectedNodes: function getSelectedNodes(e) {
      var graph = this.graph,
        originPoint = this.originPoint;
      var p1 = {
        x: e.x,
        y: e.y
      };
      var p2 = graph.getPointByCanvas(originPoint.x, originPoint.y);
      var left = Math.min(p1.x, p2.x);
      var right = Math.max(p1.x, p2.x);
      var top = Math.min(p1.y, p2.y);
      var bottom = Math.max(p1.y, p2.y);
      var nodes = [];
      var selectedNodes = [];
      var selectedEdges = [];
      graph.getNodes().forEach(function (node) {
        if (!node.isVisible()) return; // 隐藏节点不能被选中

        var bbox = node.getBBox();
        if (bbox.minX >= left && bbox.maxX <= right && bbox.minY >= top && bbox.maxY <= bottom) {
          nodes.push(node);
          selectedNodes.push(node.getModel());
          graph.setItemState(node, 'selected', true);
        }
      });
      graph.set('selectedItem', null);
      graph.emit('afterItemSelected', null);
      var selectedEdgeIds = [];
      nodes.forEach(function (node) {
        var outEdges = node.getOutEdges();
        var inEdges = node.getInEdges();
        outEdges.concat(inEdges).forEach(function (edge) {
          var model = edge.getModel();
          if (!selectedEdgeIds.includes(model.id)) {
            selectedEdgeIds.push(model.id);
            selectedEdges.push(model);
          }
        });
      });
      this.selectedNodes = selectedNodes;
      this.selectedEdges = selectedEdges;
      graph.emit('brushSelectChange', {
        selectedItems: {
          nodes: selectedNodes,
          edges: selectedEdges
        },
        select: true
      });
    },
    clearStates: function clearStates() {
      var graph = this.graph;
      (0, _graph.clearSelected)(graph);
      this.selectedNodes = [];
      this.selectedEdges = [];
      graph.emit('brushSelectChange', {
        selectedItems: {
          nodes: [],
          edges: []
        },
        select: false
      });
    },
    createBrush: function createBrush() {
      return this.graph.get('canvas').addShape('rect', {
        attrs: this.brushStyle,
        capture: false,
        name: 'brush-shape'
      });
    }
  });
}