"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = dragCanvas;
var _g = require("@antv/g6");
function dragCanvas() {
  (0, _g.registerBehavior)('dragCanvas', {
    getEvents: function getEvents() {
      return {
        dragstart: 'onMouseDown',
        drag: 'onMouseMove',
        dragend: 'onMouseUp',
        keyup: 'onMouseUp',
        keydown: 'onKeyDown'
      };
    },
    onMouseDown: function onMouseDown(e) {
      // console.log('dragCanvas: ', e);
      if (e.item || !this.keydown) return;
      var event = e.originalEvent;
      if (event && e.name !== 'touchstart' && event.button !== 0) {
        return;
      }
      this.origin = {
        x: e.clientX,
        y: e.clientY
      };
      this.dragging = false;
      this.graph.get('canvas').get('el').style.cursor = 'grabbing';
    },
    onMouseMove: function onMouseMove(e) {
      if (!this.keydown || !this.origin) return;
      this.updateViewport(e);
      this.dragging = true;
    },
    updateViewport: function updateViewport(e) {
      if (!this.origin) return;
      var clientX = +e.clientX;
      var clientY = +e.clientY;
      if (isNaN(clientX) || isNaN(clientY)) {
        return;
      }
      var dx = clientX - this.origin.x;
      var dy = clientY - this.origin.y;
      this.origin = {
        x: clientX,
        y: clientY
      };
      this.graph.translate(dx, dy);
    },
    onMouseUp: function onMouseUp() {
      this.keydown = false;
      this.origin = null;
      this.dragging = false;
      this.graph.set('canvasDragging', false);
      this.graph.get('canvas').get('el').style.cursor = 'default';
    },
    onKeyDown: function onKeyDown(e) {
      if (!e.code || this.keydown) return;
      this.keydown = e.code.toLowerCase() === 'space';
      if (this.keydown) {
        this.graph.get('canvas').get('el').style.cursor = 'grab';
      }
      this.graph.set('canvasDragging', this.keydown);
    }
  });
}