"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = edge;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _g = require("@antv/g6");
var _util = require("@antv/util");
var _polylineUtil = require("@antv/g6-element/es/edges/polyline-util");
var _router = require("@antv/g6-element/es/edges/router");
var _g6Core = require("@antv/g6-core");
var _graph = require("../util/graph");
var FlowArrow = _g.Arrow.vee(6, 8); // 自定义箭头路径

function edge() {
  (0, _g.registerEdge)('flow', {
    options: {
      style: {
        lineAppendWidth: 15,
        lineWidth: 2,
        radius: 5,
        offset: 15,
        stroke: '#A3B1BF',
        shadowBlur: 0,
        shadowColor: 'rgb(95, 149, 255)',
        endArrow: {
          path: FlowArrow,
          fillOpacity: 1
        }
      },
      labelCfg: {
        refX: 10,
        refY: 10
      },
      routeCfg: {
        gridSize: 5 // 指定精度
      }
    },
    drawShape: function drawShape(cfg, group) {
      var shapeStyle = this.getShapeStyle(cfg, group);
      if (shapeStyle.radius === 0) delete shapeStyle.radius;
      var keyShape = group.addShape('path', {
        className: 'edge-shape',
        name: 'edge-shape',
        attrs: shapeStyle
      });
      group.shapeMap['edge-shape'] = keyShape;
      var point = (0, _graph.findEdgeAnchor)(keyShape.attr('path'));
      if (point) {
        group.shapeMap['edge-anchor'] = group.addShape('rect', {
          className: 'edge-anchor',
          name: 'edge-anchor',
          attrs: {
            width: 8,
            height: 8,
            x: point.x - 4,
            y: point.y - 4,
            fill: '#fff',
            shadowBlur: 4,
            shadowColor: '#666',
            cursor: point.cursor
          },
          draggable: true,
          capture: true,
          visible: false
        });
      }
      return keyShape;
    },
    getShapeStyle: function getShapeStyle(cfg, group) {
      var _this$options, _this$options2;
      var defaultStyle = (_this$options = this.options) === null || _this$options === void 0 ? void 0 : _this$options.style;
      var strokeStyle = {
        stroke: cfg.stroke || cfg.color
      };
      var style = (0, _util.mix)({
        stroke: undefined
      }, defaultStyle, strokeStyle, cfg.style);
      cfg = this.getPathPoints ? this.getPathPoints(cfg) : cfg;
      this.radius = style.radius;
      this.offset = style.offset;
      var _cfg = cfg,
        startPoint = _cfg.startPoint,
        endPoint = _cfg.endPoint;
      var points = this.getPoints(cfg);
      var source = cfg.sourceNode;
      var target = cfg.targetNode;
      var radius = style.radius;
      var defaultRouteCfg = (_this$options2 = this.options) === null || _this$options2 === void 0 ? void 0 : _this$options2.routeCfg;
      var routeCfg = (0, _util.mix)({}, defaultRouteCfg, cfg.routeCfg);
      routeCfg.offset = cfg.offset || style.offset;
      var path = this.getPath(points, source, target, radius, routeCfg, group);
      path = this.dealPath(path, startPoint, endPoint);
      return (0, _util.mix)({}, _g6Core.BaseGlobal.defaultEdge.style, style, {
        lineWidth: cfg.size,
        path: path
      });
    },
    updateShapeStyle: function updateShapeStyle(cfg, item) {
      var _this$options3;
      if (!item.isVisible()) return;
      var group = item.getContainer();
      var strokeStyle = {
        stroke: cfg.stroke || cfg.color
      };
      var keyShape = group.shapeMap['edge-shape'] || item.getKeyShape();
      var _cfg2 = cfg,
        size = _cfg2.size,
        offset = _cfg2.offset;
      cfg = this.getPathPoints ? this.getPathPoints(cfg) : cfg;
      var _cfg3 = cfg,
        startPoint = _cfg3.startPoint,
        endPoint = _cfg3.endPoint;
      var points = this.getPoints(cfg);
      var currentAttr = keyShape.attr();
      var previousStyle = (0, _util.mix)({}, strokeStyle, currentAttr, cfg.style);
      previousStyle.offset = offset || previousStyle.offset;
      var source = cfg.sourceNode;
      var target = cfg.targetNode;
      var radius = previousStyle.radius;
      var defaultRouteCfg = (_this$options3 = this.options) === null || _this$options3 === void 0 ? void 0 : _this$options3.routeCfg;
      var routeCfg = (0, _util.mix)({}, defaultRouteCfg, cfg.routeCfg);
      routeCfg.offset = previousStyle.offset;
      var path = this.getPath(points, source, target, radius, routeCfg, group);
      path = this.dealPath(path, startPoint, endPoint);
      var style = (0, _util.mix)({}, keyShape.attr(), (0, _extends2["default"])({
        lineWidth: size,
        path: path
      }, strokeStyle), cfg.style);
      if (keyShape) {
        keyShape.attr(style);
      }
      var anchor = group.shapeMap['edge-anchor'];
      var point = (0, _graph.findEdgeAnchor)(keyShape.attr('path'));
      if (anchor && point) {
        anchor.attr({
          x: point.x - 4,
          y: point.y - 4,
          cursor: point.cursor
        });
      }
      var label = group.shapeMap['edge-label'];
      if (label) {
        label.attr({
          text: cfg.label
        });
      }
    },
    getPoints: function getPoints(cfg) {
      var startPoint = cfg.startPoint,
        endPoint = cfg.endPoint;
      var controlPoints = this.getControlPoints ? this.getControlPoints(cfg) : undefined;
      // 添加起始点
      var points = [{
        x: startPoint.x,
        y: startPoint.y
      }];
      // 添加控制点
      if (controlPoints) {
        points = points.concat(controlPoints);
      }

      // 添加结束点
      points.push({
        x: endPoint.x,
        y: endPoint.y
      });
      return points;
    },
    getPath: function getPath(points, source, target, radius, routeCfg, group) {
      var offset = routeCfg.offset,
        simple = routeCfg.simple;
      if (!offset || points.length > 2) {
        if (radius) {
          return (0, _polylineUtil.getPathWithBorderRadiusByPolyline)(points, radius);
        }
        var pathArray = [];
        (0, _util.each)(points, function (point, index) {
          if (index === 0) {
            pathArray.push(['M', point.x, point.y]);
          } else {
            pathArray.push(['L', point.x, point.y]);
          }
        });
        return pathArray;
      }

      // 未指定控制点
      var polylinePoints = simple ? (0, _polylineUtil.getPolylinePoints)(points[points.length - 1], points[0], target, source, offset) : (0, _router.pathFinder)(points[0], points[points.length - 1], source, target, routeCfg);
      if (!polylinePoints || !polylinePoints.length) return 'M0 0, L0 0';
      group.polylinePoints = polylinePoints;
      if (radius) {
        return (0, _polylineUtil.getPathWithBorderRadiusByPolyline)(polylinePoints, radius);
      }
      return _g6Core.Util.pointsToPolygon(polylinePoints);
    },
    dealPath: function dealPath(path, startPoint, endPoint) {
      if ((0, _util.isArray)(path) && path.length <= 1 || (0, _util.isString)(path) && path.indexOf('L') === -1) {
        path = 'M0 0, L0 0';
      }
      if (isNaN(startPoint.x) || isNaN(startPoint.y) || isNaN(endPoint.x) || isNaN(endPoint.y)) {
        path = 'M0 0, L0 0';
      }
      return path;
    },
    setState: function setState(name, value, item) {
      var group = item === null || item === void 0 ? void 0 : item.getContainer();
      var anchor = group.shapeMap['edge-anchor'];
      var keyShape = group.shapeMap['edge-shape'];
      switch (name) {
        case 'hover':
          if (value) {
            keyShape.attr({
              shadowBlur: 10
            });
            anchor === null || anchor === void 0 ? void 0 : anchor.show();
          } else {
            keyShape.attr({
              shadowBlur: 0,
              lineWidth: 2
            });
            anchor === null || anchor === void 0 ? void 0 : anchor.hide();
          }
          break;
        case 'selected':
          if (value) {
            keyShape.attr({
              lineWidth: 3,
              shadowBlur: 10
            });
          } else {
            keyShape.attr({
              shadowBlur: 0,
              lineWidth: 2
            });
            anchor === null || anchor === void 0 ? void 0 : anchor.hide();
          }
          break;
        case 'selected-view':
          if (value) {
            keyShape.attr({
              lineWidth: 3,
              shadowBlur: 10
            });
          } else {
            keyShape.attr({
              shadowBlur: 0,
              lineWidth: 2
            });
          }
          anchor === null || anchor === void 0 ? void 0 : anchor.hide();
          break;
      }
    }
  }, 'polyline');
}