"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = node;
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _defaultStyle = _interopRequireDefault(require("../util/defaultStyle"));
var _g = require("@antv/g6");
/**
 * format the string
 * @param {string} str The origin string
 * @param {number} maxWidth max width
 * @param {number} fontSize font size
 * @return {string} the processed result
 */
var fittingString = function fittingString(str, maxWidth) {
  var fontSize = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 12;
  var currentWidth = 0;
  var res = str;
  var pattern = new RegExp("[\u4E00-\u9FA5]+"); // distinguish the Chinese characters and letters
  str.split('').forEach(function (letter, i) {
    if (currentWidth > maxWidth) return;
    if (pattern.test(letter)) {
      // Chinese characters
      currentWidth += fontSize;
    } else {
      // get the width of single letter according to the fontSize
      currentWidth += _g.Util.getLetterWidth(letter, fontSize);
    }
    if (currentWidth > maxWidth) {
      res = "".concat(str.substr(0, i), "\n").concat(str.substr(i));
    }
  });
  return res;
};
function node() {
  (0, _g.registerNode)('base-node', {
    options: {
      style: {
        fill: '#f9f9f9',
        stroke: '#bbb',
        cursor: 'default'
      },
      stateStyles: {
        selected: {
          fill: '#eee'
        },
        hover: {
          cursor: _defaultStyle["default"].cursor.hoverNode
        }
      }
    },
    drawAnchor: function drawAnchor(group, bbox) {
      var isCreate = false;
      if (!group.anchorShapes) {
        group.anchorShapes = [];
        isCreate = true;
      }
      var anchorStyle = {
        r: 4,
        lineWidth: 1,
        stroke: '#1890FF',
        fill: '#fff',
        cursor: 'pointer'
      };
      this.getAnchorPoints().forEach(function (p, index) {
        var position = {
          x: bbox.minX + bbox.width * p[0],
          y: bbox.minY + bbox.height * p[1]
        };
        var marker = group.shapeMap["marker-".concat(index)];
        var anchor = group.shapeMap["anchor-".concat(index)];
        if (!marker) {
          marker = group.addShape('marker', {
            attrs: (0, _extends2["default"])({}, position, {
              symbol: 'circle',
              r: 12,
              fill: '#1890FF',
              shadowBlur: 10,
              fillOpacity: 0.25,
              cursor: 'pointer'
            }),
            index: index,
            name: 'marker',
            draggable: true,
            capture: true,
            visible: false
          });
          group.shapeMap["marker-".concat(index)] = marker;
        } else {
          marker.attr(position);
        }
        if (!anchor) {
          anchor = group.addShape('marker', {
            attrs: (0, _extends2["default"])({}, position, {
              symbol: 'circle'
            }, anchorStyle),
            index: index,
            draggable: true,
            capture: true,
            name: 'anchor',
            visible: false
          });
          group.shapeMap["anchor-".concat(index)] = anchor;
        } else {
          anchor.attr(position);
        }
        if (isCreate) {
          group.anchorShapes.push([anchor, marker]);
        }
      });
      if (isCreate) {
        // 显示锚点
        group.showAnchor = function (g, show) {
          if (show) {
            g.anchorShapes.forEach(function (_ref) {
              var _ref2 = (0, _slicedToArray2["default"])(_ref, 1),
                anchor = _ref2[0];
              return anchor.show();
            });
          } else {
            g.anchorShapes.forEach(function (_ref3) {
              var _ref4 = (0, _slicedToArray2["default"])(_ref3, 1),
                anchor = _ref4[0];
              return anchor.hide();
            });
          }
        };
        // 显示可连接的热锚点
        group.showHotpot = function (g, show) {
          if (show) {
            g.anchorShapes.forEach(function (_ref5) {
              var _ref6 = (0, _slicedToArray2["default"])(_ref5, 2),
                anchor = _ref6[0],
                marker = _ref6[1];
              anchor.show();
              marker.show();
            });
          } else {
            g.anchorShapes.forEach(function (_ref7) {
              var _ref8 = (0, _slicedToArray2["default"])(_ref7, 2),
                anchor = _ref8[0],
                marker = _ref8[1];
              anchor.hide();
              marker.hide();
            });
          }
        };
        group.hideAnchor = function () {
          group.anchorShapes.forEach(function (_ref9) {
            var _ref10 = (0, _slicedToArray2["default"])(_ref9, 2),
              anchor = _ref10[0],
              marker = _ref10[1];
            anchor.hide();
            marker.hide();
          });
        };
      }
    },
    drawBg: function drawBg(group, bbox) {
      var _bg;
      var bg = group.shapeMap['main-box-bg'];
      var style = {
        x: bbox.x - 4,
        y: bbox.y - 4,
        width: bbox.width + 8,
        height: bbox.height + 8
      };
      if (!bg) {
        bg = group.addShape('rect', {
          attrs: (0, _extends2["default"])({}, style, {
            stroke: '#428fff',
            lineDash: [3, 2],
            fill: 'rgba(0,0,0,0)'
          }),
          visible: false,
          draggable: true,
          capture: true,
          name: 'main-box-bg'
        });
      } else {
        bg.attr(style);
      }
      (_bg = bg) === null || _bg === void 0 ? void 0 : _bg.toBack();
      group.shapeMap['main-box-bg'] = bg;
    },
    drawKeyShape: function drawKeyShape(cfg, group, style) {
      var keyShape = group.shapeMap['main-box'];
      if (!keyShape) {
        keyShape = group === null || group === void 0 ? void 0 : group.addShape(this.shapeType, {
          attrs: (0, _extends2["default"])({}, style),
          draggable: true,
          capture: true,
          name: 'main-box'
        });
      } else {
        keyShape.attr(style);
      }
      group.shapeMap['main-box'] = keyShape;
      return keyShape;
    },
    drawShape: function drawShape(cfg, group) {
      var _this$options;
      var style = this.getShapeStyle(cfg);
      var keyShape = this.drawKeyShape(cfg, group, style);
      var bbox = keyShape.getBBox();
      this.drawBg(group, bbox);
      this.drawAnchor(group, bbox);
      this.drawControlPoints(cfg, group, bbox);
      group.minSize = ((_this$options = this.options) === null || _this$options === void 0 ? void 0 : _this$options.minSize) || [90, 40];
      return keyShape;
    },
    drawControlPoints: function drawControlPoints(cfg, group, bbox) {
      var isCreate = false;
      if (!group.resizeShapes) {
        group.resizeShapes = [];
        isCreate = true;
      }
      var offsets = [[-8, -8], [0, -8], [-8, 0], [0, 0]];
      this.getResizePoints(cfg).forEach(function (p, index) {
        var cursor = 'default';
        if (p[0] === 0 && p[1] === 0) {
          cursor = 'nwse-resize';
        } else if (p[0] === 0 && p[1] === 1) {
          cursor = 'nesw-resize';
        } else if (p[0] === 1 && p[1] === 0) {
          cursor = 'nesw-resize';
        } else if (p[0] === 1 && p[1] === 1) {
          cursor = 'nwse-resize';
        }
        var position = {
          x: bbox.minX + bbox.width * p[0] + offsets[index][0],
          y: bbox.minY + bbox.height * p[1] + offsets[index][1]
        };
        var resizePoint = group.shapeMap["resize-point-".concat(index)];
        if (!resizePoint) {
          resizePoint = group.addShape('rect', {
            attrs: (0, _extends2["default"])({}, position, {
              width: 8,
              height: 8,
              fill: '#fff',
              shadowBlur: 4,
              shadowColor: '#666',
              cursor: cursor
            }),
            index: index,
            name: 'resize-point',
            draggable: true,
            capture: true,
            visible: false
          });
          group.shapeMap["resize-point-".concat(index)] = resizePoint;
        } else {
          resizePoint.attr(position);
        }
        if (isCreate) {
          group.resizeShapes.push(resizePoint);
        }
      });
      group.showResize = function (g, show) {
        if (show) {
          g.resizeShapes.forEach(function (resizePoint) {
            resizePoint.toFront();
            resizePoint.show();
          });
        } else {
          g.resizeShapes.forEach(function (resizePoint) {
            resizePoint.toBack();
            resizePoint.hide();
          });
        }
      };
    },
    setState: function setState(name, value, item) {
      var group = item.getContainer();
      var bg = group.shapeMap['main-box-bg'];
      if (name === 'show-anchor') {
        if (value) {
          group.showAnchor(group, true);
        } else {
          group.hideAnchor();
        }
      } else if (/selected/.test(name)) {
        if (value) {
          group.hideAnchor();
          name === 'selected' && group.showResize(group, true);
          bg === null || bg === void 0 ? void 0 : bg.show();
        } else {
          name === 'selected' && group.showResize(group, false);
          bg === null || bg === void 0 ? void 0 : bg.hide();
        }
      } else if (name === 'hover') {
        var rect = group.getChildByIndex(0);
        if (value) {
          rect.attr('cursor', this.options.stateStyles.hover.cursor);
        } else {
          rect.attr('cursor', this.options.style.cursor);
        }
      }
    },
    // 拖动改变Node 大小的控制点
    getResizePoints: function getResizePoints(cfg) {
      return /START|END/.test(cfg.clazz) ? [] : [[0, 0],
      // left top
      [1, 0],
      // right top
      [0, 1],
      // left bottom
      [1, 1] // right bottom
      ];
    },
    getAnchorPoints: function getAnchorPoints() {
      return [[0.5, 0],
      // top 0
      [1, 0.5],
      // right 1
      [0.5, 1],
      // bottom 2
      [0, 0.5],
      // left 3
      [0, 0],
      // top-left
      [1, 0],
      // top-left
      [0, 1],
      // left bottom
      [1, 1] // right bottom
      ];
    },
    update: function update(cfg, item, updateType) {
      var group = item.getContainer();
      var keyShape = group.shapeMap['main-box'];
      switch (updateType) {
        case 'bbox':
        case 'bbox|label':
          {
            this.drawKeyShape(cfg, group, this.getShapeStyle(cfg));
            var _bbox = keyShape.getBBox();
            this.drawBg(group, _bbox);
            this.drawAnchor(group, _bbox);
            this.drawControlPoints(cfg, group, _bbox);
            var label = group.shapeMap['node-label'];
            if (label) label.attr({
              text: fittingString(cfg.label, _bbox.width)
            });
          }
          break;
        case 'style|label':
          {
            var _bbox2 = keyShape.getBBox();
            var _label = group.shapeMap['node-label'];
            if (_label) _label.attr({
              text: fittingString(cfg.label, _bbox2.width - 10)
            });
          }
          break;
        default:
          break;
      }
    }
  }, 'single-shape');
}