"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.syncNodeGroup = exports.shouldConnect = exports.setSelected = exports.findEdgeAnchor = exports.clearSelected = exports.beforeAddNode = exports.beforeAddEdge = void 0;
exports.uniqIdByDate = uniqIdByDate;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _interface = require("../interface");
var clearSelected = function clearSelected(graph) {
  var emit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var state = 'selected';
  var selected = graph.findAllByState('node', state);
  selected.forEach(function (node) {
    graph.setItemState(node, state, false);
  });
  selected = graph.findAllByState('edge', state);
  selected.forEach(function (edge) {
    graph.setItemState(edge, state, false);
  });
  if (emit) {
    graph.set('selectedItem', null);
    graph.emit('afterItemSelected', null);
  }
};
exports.clearSelected = clearSelected;
var setSelected = function setSelected(graph, items) {
  items.forEach(function (item) {
    graph.setItemState(item.id, 'selected', true);
  });
};
exports.setSelected = setSelected;
function uniqIdByDate(prefix) {
  return "".concat(prefix ? "".concat(prefix, "-") : '').concat((Math.floor(Math.random() * 100) + Date.now()).toString(32));
}

/**
 * line:
 * ['Q', -215, -156.5, -215, -161.5]
 * ['L', -215, -161.5]
 * ['L', -215, -195]
 * ['Q', -215, -200, -220, -200]
 * @param paths
 */
var findEdgeAnchor = function findEdgeAnchor(paths) {
  var startIndex = paths.findIndex(function (point) {
    return point[0] === 'Q';
  });
  var endIndex = -1;
  if (startIndex > -1) {
    endIndex = paths.slice(startIndex + 1).findIndex(function (point) {
      return point[0] === 'Q';
    });
  } else {
    return null;
  }
  var line = paths.slice(startIndex, startIndex + 1 + endIndex + 1);
  if (line.length <= 1) return null;
  var pointX = line[0][1];
  var pointY = line[0][2];
  // 点坐标变化不是垂直 就是 水平
  if (line.every(function (point) {
    return point[1] === pointX;
  })) {
    return {
      x: pointX,
      y: (line[1][2] + line[2][2]) / 2,
      cursor: 'col-resize'
    };
  }
  return {
    x: (line[1][1] + line[2][1]) / 2,
    y: pointY,
    cursor: 'row-resize'
  };
};
exports.findEdgeAnchor = findEdgeAnchor;
var recursiveUpdate = function recursiveUpdate(graph, source, cfg) {
  if (source.type === _interface.WorkFlowNodeEnum.END) return;
  var edges = graph.findById(source.id).getOutEdges();
  edges.forEach(function (edge) {
    var target = edge.getTarget();
    var model = target.getModel();
    if (model.parentGroupId || model.groupId) return;
    console.log('recursiveUpdate: ', model, target);
    graph.update(target, (0, _extends2["default"])({}, cfg), false);
    recursiveUpdate(graph, model, cfg);
  });
};
var syncNodeGroup = function syncNodeGroup(graph, sourceModel, targetModel) {
  var cfg = {
    groupId: targetModel.groupId
  };
  console.log('syncNodeGroup： ', sourceModel, targetModel);
  if (sourceModel.clazz === _interface.WorkFlowNodeEnum.END || targetModel.parentGroupId) return;

  // 标识流程第一步
  if (sourceModel.clazz === _interface.WorkFlowNodeEnum.START) {
    cfg.isProcFirst = true;
  }
  if (!targetModel.groupId) {
    cfg.groupId = sourceModel.groupId;
  }

  // 如果自身有groupId, 并且和源节点不是同组的节点 说明目标节点是子流程节点,
  if (targetModel.groupId && targetModel.groupId !== sourceModel.groupId) {
    cfg.parentGroupId = sourceModel.groupId;
  }

  // 如果是同组节点，并且是子流程节点
  if (cfg.groupId === sourceModel.groupId && sourceModel.parentGroupId) {
    cfg.parentGroupId = sourceModel.parentGroupId;
  }
  graph.update(targetModel.id, cfg, false);
  if (cfg.groupId || cfg.parentGroupId) {
    recursiveUpdate(graph, targetModel, {
      groupId: cfg.groupId,
      parentGroupId: cfg.parentGroupId
    });
  }
};
exports.syncNodeGroup = syncNodeGroup;
var beforeAddNode = function beforeAddNode(graph, addModel) {
  // 将一个开始节点 当做一个流程分组 用来区分不同子流程
  if (addModel.clazz === _interface.WorkFlowNodeEnum.START) {
    addModel.groupId = Date.now();
  } else {
    delete addModel.groupId;
  }
  if ([_interface.WorkFlowNodeEnum.PARALLEL_SIGN, _interface.WorkFlowNodeEnum.SERIAL_MULTI_SIGN].includes(addModel.clazz)) {
    addModel.timeStmp = new Date().getTime();
  }
  addModel.defKey = addModel.id;
  addModel.defName = addModel.label;
  addModel.defCategory = addModel.clazz;
  delete addModel.parentGroupId;
};
exports.beforeAddNode = beforeAddNode;
var beforeAddEdge = function beforeAddEdge(graph, source, target, addModel) {
  var sourceModel = source.getModel();
  var targetModel = target.getModel();
  addModel.flowName = '';
  addModel.flowType = '';
  // 开始节点
  if (addModel.sourceType === _interface.WorkFlowNodeEnum.START) {
    addModel.flowName = '提交';
    addModel.flowType = '1';
  }
  addModel.sourceTaskDefKey = sourceModel.defKey;
  addModel.tarTaskDefKey = targetModel.defKey;

  // 服务任务入线和出线配置相同
  if (sourceModel.clazz === _interface.WorkFlowNodeEnum.SERVICE) {
    var edge = source.getInEdges()[0];
    if (edge) {
      var model = edge.getModel();
      addModel.flowName = model.flowName;
      addModel.flowType = model.flowType;
      addModel.color = model.color;
      addModel.stroke = model.stroke;
    }
  }
  syncNodeGroup(graph, sourceModel, targetModel);
};
exports.beforeAddEdge = beforeAddEdge;
var shouldConnect = function shouldConnect(source, target) {
  var sourceOutEdges = source.getOutEdges();
  var sourceModel = source.getModel();
  var targetInEdges = target.getInEdges();
  var targetModel = target.getModel();
  switch (sourceModel.clazz) {
    case _interface.WorkFlowNodeEnum.START:
      if (targetModel.clazz !== _interface.WorkFlowNodeEnum.NORMAL) return false;

      // 子流程开始节点 只能链接 没有分组 或者 同组的节点
      if (sourceModel.parentGroupId && targetModel.groupId && sourceModel.groupId !== targetModel.groupId) return false;
      break;
    case _interface.WorkFlowNodeEnum.END:
      if (!sourceModel.parentGroupId) return false;
      if (targetModel.clazz === _interface.WorkFlowNodeEnum.START) return false;
      if (sourceModel.groupId === targetModel.groupId) return false;
      if (sourceModel.parentGroupId !== targetModel.groupId) return false;
      break;
    case _interface.WorkFlowNodeEnum.NORMAL:
      if (!sourceModel.groupId) return false;
      if (targetModel.clazz === _interface.WorkFlowNodeEnum.START) return !sourceModel.parentGroupId && sourceModel.groupId !== targetModel.groupId;
      if (targetModel.groupId && sourceModel.groupId !== targetModel.groupId) return false;
      break;
    case _interface.WorkFlowNodeEnum.SERVICE:
      if (sourceOutEdges.length > 0) return false;
      break;
  }

  // 只能有一条出线
  if ([_interface.WorkFlowNodeEnum.START, _interface.WorkFlowNodeEnum.SERVICE, _interface.WorkFlowNodeEnum.END].includes(sourceModel.clazz)) {
    return sourceOutEdges.length === 0;
  }

  // 只能有一条入线
  if ([_interface.WorkFlowNodeEnum.START, _interface.WorkFlowNodeEnum.SERVICE].includes(targetModel.clazz)) {
    return targetInEdges.length === 0;
  }
  return true;
};
exports.shouldConnect = shouldConnect;