"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.ErrTips = void 0;
/** 表单元素验证函数
 */
var REGEXP = {
  email: {
    reg: /^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/,
    errMsg: '请输入正确的邮箱地址'
  },
  link: {
    reg: /(https?|ftp|file):\/\/[-A-Za-z0-9+&@#/%?=~_|!:,.;]+[-A-Za-z0-9+&@#/%=~_|]/,
    errMsg: '请输入正确的链接'
  }
};
var ErrTips = {
  TextInput: '请填写',
  TextArea: '请填写',
  NumberInput: '请填写',
  SearchInput: '请选择',
  RadioGroup: '请选择',
  CheckBoxGroup: '请选择',
  SelectInput: '请选择',
  MultiStageSelect: '请选择',
  DatePicker: '请选择',
  DateRangePicker: '请选择',
  TimePicker: '请选择',
  TimeRangePicker: '请选择',
  FileUpload: '请上传',
  ImageShow: '请上传',
  VideoShow: '请上传',
  RichText: '请填写',
  ChooseUser: '请选择',
  ChooseGroup: '请选择',
  TabPage: '请填写完成',
  TableWidget: '请填写完成',
  AttachList: '请上传'
};
exports.ErrTips = ErrTips;
var defaultValidate = function defaultValidate(fieldValue, config, required) {
  var valid = true;
  if (required) {
    valid = typeof fieldValue !== "undefined" ? fieldValue !== '' : false;
  }
  return valid;
};
var dateRangeValidate = function dateRangeValidate(fieldValue, config, required) {
  var valid = true;
  if (required) {
    valid = fieldValue ? fieldValue.length === 2 : false;
  }
  return valid;
};
var travelTable = function travelTable(tableHead, tableBody) {
  tableHead.forEach(function (head) {
    if (head.required) {
      tableBody.forEach(function (tr) {
        if (tr[head.fieldKey] === '') {
          throw Error("\u8868\u683C\u5217\uFF1A".concat(head.fieldName, " \u5FC5\u586B\uFF0C\u8BF7\u586B\u5199\u5B8C\u6574"));
        }
      });
    }
  });
};
var arrayValidate = function arrayValidate(fieldValue, config, required) {
  var valid = true;
  if (required) {
    valid = fieldValue && fieldValue.length > 0;
  }
  return valid;
};
var ValidateFunc = {
  TextInput: function TextInput(fieldValue, config, required) {
    var setList = config.setList;
    var valid = true;
    if (required) {
      valid = !!fieldValue;
    }
    if (REGEXP[config.dataType] && !REGEXP[config.dataType].reg.test(fieldValue)) {
      return new Error(REGEXP[config.dataType].errMsg);
    }
    if (setList.indexOf('isTitle') >= 0) {
      valid = fieldValue && fieldValue.trim() === '' ? new Error('作为标题，输入的内容不能全为不可见字符') : !!fieldValue;
    }
    return valid;
  },
  TextArea: function TextArea(fieldValue, config, required) {
    var valid = true;
    if (required) {
      valid = fieldValue && fieldValue.trim() === '' ? new Error('输入的内容不能为不可见字符') : !!fieldValue;
    }
    return valid;
  },
  NumberInput: function NumberInput(fieldValue, config, required) {
    var valid = true;
    if (required) {
      valid = typeof fieldValue === 'number' ? true : !!fieldValue;
    }
    return valid;
  },
  SearchInput: defaultValidate,
  SelectInput: defaultValidate,
  CheckBoxGroup: arrayValidate,
  RadioGroup: defaultValidate,
  MultiStageSelect: arrayValidate,
  DatePicker: defaultValidate,
  DateRangePicker: dateRangeValidate,
  TimePicker: defaultValidate,
  TimeRangePicker: dateRangeValidate,
  TableWidget: function TableWidget(tableBody, config, required) {
    if (required && tableBody.length === 0) {
      return new Error('请添加数据');
    } else {
      try {
        travelTable(config.tableHead, tableBody);
        return true;
      } catch (e) {
        return e;
      }
    }
  },
  ChooseUser: defaultValidate,
  ChooseGroup: defaultValidate,
  AttachList: arrayValidate
};
var _default = ValidateFunc;
exports["default"] = _default;