import { registerBehavior } from "@antv/g6";
export default function clickSelected() {
  registerBehavior('clickSelected', {
    getEvents: function getEvents() {
      return {
        'node:click': 'onClick',
        'edge:click': 'onClick',
        'canvas:click': 'clearSelected'
      };
    },
    onClick: function onClick(e) {
      var graph = this.graph;
      var mode = graph.getCurrentMode();
      var item = e.item;
      item.toFront();
      var itemId = item.get('id');
      if (mode === 'view') {
        graph.emit('itemClicked', itemId);
        return;
      }
      var state = mode === 'edit' ? 'selected' : 'selected-view';
      if (graph.get('selectedItem') !== itemId) {
        this.clearSelected();
        graph.setItemState(item, state, true);
        graph.set('selectedItem', itemId);
        graph.emit('afterItemSelected', itemId);
      }
    },
    clearSelected: function clearSelected() {
      var _this = this;
      var mode = this.graph.getCurrentMode();
      var state = mode === 'edit' ? 'selected' : 'selected-view';
      var selected = this.graph.findAllByState('node', state);
      selected.forEach(function (node) {
        _this.graph.setItemState(node, state, false);
      });
      selected = this.graph.findAllByState('edge', state);
      selected.forEach(function (edge) {
        _this.graph.setItemState(edge, state, false);
      });
      this.graph.set('selectedItem', null);
      this.graph.emit('afterItemSelected', null);
    }
  });
}