import _extends from "@babel/runtime/helpers/esm/extends";
import { registerBehavior } from '@antv/g6';
import { uniqIdByDate } from '../util/graph';
import { WorkFlowNodeEnum } from "../interface";
export default function dragAddEdge() {
  registerBehavior('dragAddEdge', {
    getDefaultCfg: function getDefaultCfg() {
      return {
        origin: null
      };
    },
    getEvents: function getEvents() {
      return {
        'anchor:dragstart': 'onDragStart',
        'anchor:drag': 'onDrag',
        'anchor:dragend': 'onDragEnd',
        'anchor:dragenter': 'onDragEnter',
        'marker:dragenter': 'onDragEnter',
        'anchor:dragleave': 'onDragLeave',
        'marker:dragleave': 'onDragLeave'
      };
    },
    sameNode: function sameNode(e) {
      var _e$item;
      var origin = this.origin;
      return ((_e$item = e.item) === null || _e$item === void 0 ? void 0 : _e$item.get('id')) === origin.sourceNode.get('id');
    },
    onDragLeave: function onDragLeave(e) {
      if (!this.origin) {
        return;
      }
      if (!this.sameNode(e)) {
        this.origin.targetNode = null;
        this.origin.targetAnchor = null;
      }
    },
    onDragEnter: function onDragEnter(e) {
      var origin = this.origin;
      if (!origin) {
        return;
      }
      if (!this.sameNode(e)) {
        origin.targetNode = e.item;
        origin.targetAnchor = e.target.get('index');
      }
    },
    onDragEnd: function onDragEnd(e) {
      var graph = this.graph;
      var origin = this.origin;
      if (!origin) {
        return;
      }
      var delegateShape = graph.get('edgeDelegate');
      if (delegateShape) {
        delegateShape.remove(true);
        graph.set('edgeDelegate', null);
      }
      this.updateEdge(e, true);
      graph.setItemState(origin.sourceNode, 'show-anchor', false);
      this.target = null;
      this.origin = null;
      graph.set('onDragEdge', false);
    },
    onDrag: function onDrag(e) {
      if (!this.origin) {
        return;
      }
      this.updateEdge(e);
    },
    updateEdgeDelegate: function updateEdgeDelegate(x, y) {
      var graph = this.graph;
      var origin = this.origin;
      var edgeShape = graph.get('edgeDelegate');
      if (!edgeShape) {
        var parent = graph.get('group');
        edgeShape = parent.addShape('line', {
          attrs: {
            x1: origin.x,
            y1: origin.y,
            x2: x,
            y2: y,
            stroke: '#1890FF',
            lineDash: [4, 4],
            lineWidth: 1
          }
        });
        edgeShape.set('capture', false);
        graph.set('edgeDelegate', edgeShape);
      }
      edgeShape.attr({
        x2: x,
        y2: y
      });
    },
    updateEdge: function updateEdge(e, force) {
      var x = e.x,
        y = e.y;
      if (!force) {
        this.updateEdgeDelegate(x, y);
        return;
      }
      this.addEdge();
      this.clearAllAnchor();
    },
    onDragStart: function onDragStart(e) {
      var _e$item2;
      e.stopPropagation();
      var graph = this.graph;
      var node = e.target.getParent().get('item');
      var group = (_e$item2 = e.item) === null || _e$item2 === void 0 ? void 0 : _e$item2.getContainer();
      var anchorIndex = e.target.get('index');
      var point = node.getAnchorPoints()[anchorIndex];
      this.target = e.target;
      this.origin = {
        x: point.x,
        y: point.y,
        sourceNode: node,
        sourceAnchor: anchorIndex,
        targetNode: null,
        targetAnchor: null
      };
      group.showAnchor(group, true);
      this.dragEdgeBeforeShowAnchor();
      graph.set('onDragEdge', true);
    },
    dragEdgeBeforeShowAnchor: function dragEdgeBeforeShowAnchor() {
      var graph = this.graph;
      var origin = this.origin;
      if (!(origin !== null && origin !== void 0 && origin.sourceNode)) return;
      var sourceNode = origin.sourceNode;
      var showHotpot = function showHotpot(group) {
        group.showHotpot(group, true);
      };
      var shouldConnect = graph.get('shouldConnect');
      graph.getNodes().forEach(function (target) {
        // 需要过滤出可以连线的节点
        var group = target.getContainer();

        // 排除自身
        if (sourceNode.get('id') === target.get('id')) return;
        var sourceOutEdges = sourceNode.getOutEdges();

        // 当前已经存在一条连线
        if (sourceOutEdges.find(function (edge) {
          return edge.get('target').get('id') === target.get('id');
        })) return;

        // 自定义连线逻辑
        if (shouldConnect && !shouldConnect(sourceNode, target)) return;
        showHotpot(group);
      });
    },
    addEdge: function addEdge() {
      var graph = this.graph;
      var origin = this.origin;
      if (origin.targetNode) {
        var _origin$targetNode$ge;
        var sourceModel = origin.sourceNode.getModel();
        // 如果source节点是服务任务，需要同步节点入线的配置
        var syncModel = {};
        if (sourceModel.clazz === WorkFlowNodeEnum.SERVICE) {
          var inEdge = origin.sourceNode.getInEdges()[0];
          if (inEdge) {
            syncModel = inEdge.getModel();
          }
        }
        var _addEdge = _extends({}, syncModel, {
          clazz: 'flow',
          source: origin.sourceNode.get('id'),
          sourceAnchor: origin.sourceAnchor,
          sourceType: sourceModel.clazz,
          target: origin.targetNode.get('id'),
          targetAnchor: origin.targetAnchor,
          targetType: (_origin$targetNode$ge = origin.targetNode.getModel()) === null || _origin$targetNode$ge === void 0 ? void 0 : _origin$targetNode$ge.clazz,
          id: uniqIdByDate('flow')
        });
        var beforeAddEdge = graph.get('beforeAddEdge');
        if (beforeAddEdge) {
          beforeAddEdge(graph, origin.sourceNode, origin.targetNode, _addEdge);
        }
        console.log('addEdge:', _addEdge);
        var item = graph.add('edge', _addEdge);
        graph.emit('afterAddEdge', {
          item: item,
          model: _addEdge
        });
      }
    },
    clearAllAnchor: function clearAllAnchor() {
      this.graph.getNodes().forEach(function (node) {
        var group = node.getContainer();
        group.hideAnchor();
      });
    }
  });
}