import _extends from "@babel/runtime/helpers/esm/extends";
import { registerBehavior } from '@antv/g6';
import { uniqIdByDate } from '../util/graph';
import { getNodePrefix } from "../util/clazz";
export default function dragAddNode() {
  registerBehavior('dragAddNode', {
    getEvents: function getEvents() {
      return {
        'canvas:mouseup': 'onMouseUp'
      };
    },
    onMouseUp: function onMouseUp(e) {
      var graph = this.graph;
      if (graph.get('onDragAddNode')) {
        var wrap = graph.get('container');
        var rect = wrap.getBoundingClientRect();

        // 坐标在视图范围内
        if (rect.left <= e.clientX && rect.right >= e.clientX && rect.top <= e.clientY && rect.bottom >= e.clientY) {
          var _p = graph.getPointByClient(e.clientX, e.clientY);
          this.addNode(_p);
        }
      }
    },
    addNode: function addNode(p) {
      var graph = this.graph;
      if (graph.get('onDragAddNode')) {
        var addModel = graph.get('addModel');
        var shouldAddNode = graph.get('shouldAddNode');
        if (shouldAddNode && !shouldAddNode(graph, addModel)) {
          return;
        }
        var id = uniqIdByDate(getNodePrefix(addModel.clazz));
        var x = p.x,
          y = p.y;
        var size = addModel.size.split('*').map(Number);
        var model = _extends({}, addModel, {
          x: x,
          y: y,
          size: size,
          id: id
        });
        var beforeAddNode = graph.get('beforeAddNode');
        beforeAddNode && beforeAddNode(graph, model);
        graph.add('node', model, true);
        graph.set('onDragAddNode', false);
      }
    }
  });
}