import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { registerBehavior } from '@antv/g6';
export default function dragResize() {
  registerBehavior('dragResize', {
    getDefaultCfg: function getDefaultCfg() {
      return {
        origin: null
      };
    },
    getEvents: function getEvents() {
      return {
        'resize-point:dragstart': 'onDragStart',
        'resize-point:drag': 'onDrag',
        'resize-point:dragend': 'onDragEnd'
      };
    },
    onDragStart: function onDragStart(e) {
      var _e$item;
      e.stopPropagation();
      var anchorIndex = e.target.get('index');
      var group = (_e$item = e.item) === null || _e$item === void 0 ? void 0 : _e$item.getContainer();
      var _ref = group.minSize,
        _ref2 = _slicedToArray(_ref, 2),
        minWidth = _ref2[0],
        minHeight = _ref2[1];
      this.origin = {
        x: e.x,
        y: e.y,
        sourceNode: e.item,
        sourceAnchor: anchorIndex,
        minWidth: minWidth,
        minHeight: minHeight
      };
    },
    onDrag: function onDrag(e) {
      if (!this.origin) {
        return;
      }
      this.resizeNode(e);
    },
    resizeNode: function resizeNode(e) {
      var stack = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var _ref3 = this.origin,
        x = _ref3.x,
        y = _ref3.y,
        sourceNode = _ref3.sourceNode,
        sourceAnchor = _ref3.sourceAnchor,
        minHeight = _ref3.minHeight,
        minWidth = _ref3.minWidth;
      var model = sourceNode.getModel();
      var size = model.size;
      var currentWidth = size[0];
      var currentHeight = size[1];
      var addWidth = e.x - x;
      var addHeight = e.y - y;
      var width = currentWidth;
      var height = currentHeight;
      var pointIndex = sourceAnchor;
      if (pointIndex === 0) {
        width = currentWidth - addWidth;
        height = currentHeight - addHeight;
      } else if (pointIndex === 1) {
        width = currentWidth + addWidth;
        height = currentHeight - addHeight;
      } else if (pointIndex === 2) {
        width = currentWidth - addWidth;
        height = currentHeight + addHeight;
      } else if (pointIndex === 3) {
        width = currentWidth + addWidth;
        height = currentHeight + addHeight;
      }
      this.graph.update(sourceNode, {
        size: [Math.max(width, minWidth), Math.max(height, minHeight)]
      }, stack);
      this.origin.x = e.x;
      this.origin.y = e.y;
    },
    onDragEnd: function onDragEnd(e) {
      if (!this.origin) {
        return;
      }
      this.resizeNode(e, true);
      this.origin = null;
    }
  });
}