import _extends from "@babel/runtime/helpers/esm/extends";
// 键盘事件
import { registerBehavior } from '@antv/g6';
import { clone } from '@antv/util';
import { clearSelected, setSelected, uniqIdByDate } from '../util/graph';
var isMac = !!window.navigator.userAgent.match(/Mac/);
var keyboardActions = {
  undo: {
    controlKey: !isMac,
    metaKey: isMac,
    shortcut: ['z']
  },
  redo: {
    controlKey: !isMac,
    metaKey: isMac,
    shiftKey: true,
    shortcut: ['z']
  },
  "delete": {
    shortcut: ['delete', 'backspace']
  },
  copy: {
    controlKey: !isMac,
    metaKey: isMac,
    shortcut: ['c']
  },
  paste: {
    controlKey: !isMac,
    metaKey: isMac,
    shortcut: ['v']
  },
  zoomIn: {
    shiftKey: true,
    shortcut: ['+', '=']
  },
  zoomOut: {
    shiftKey: true,
    shortcut: ['-', '_']
  }
};
function capitalize(str) {
  return str.split('').map(function (item, index) {
    return index === 0 ? item.toUpperCase() : item;
  }).join('');
}
var keyboardMapActions = Object.create(null);
var keyActions = Object.keys(keyboardActions);
keyActions.forEach(function (action) {
  var item = keyboardActions[action];
  item.shortcut.forEach(function (key) {
    var shortcut = '';
    if (item.metaKey) {
      shortcut += 'meta ';
    }
    if (item.controlKey) {
      shortcut += 'ctrl ';
    }
    if (item.shiftKey) {
      shortcut += 'shift ';
    }
    shortcut += key;
    keyboardMapActions[shortcut] = action;
  });
});
var shortcut = '';
export default function keyboard() {
  registerBehavior('keyboard', {
    getEvents: function getEvents() {
      return {
        keydown: 'onKeydown',
        brushSelectChange: 'onSelected',
        'canvas:mouseleave': 'onCanvasLeave',
        'canvas:mouseenter': 'onCanvasFocus',
        'delete': 'executeDelete',
        'executeCommand': 'execute'
      };
    },
    onKeydown: function onKeydown(e) {
      var focus = this.graph.get('focusGraphWrapper');
      if (!e.key || !focus || e.repeat) return;
      if (e.metaKey) {
        shortcut += 'meta ';
      }
      if (e.controlKey || e.ctrlKey) {
        shortcut += 'ctrl ';
      }
      if (e.shiftKey) {
        shortcut += 'shift ';
      }
      if (!['meta', 'ctrl', 'control', 'shift'].includes(e.key.toLowerCase())) {
        shortcut += e.key.toLowerCase();
      }
      if (keyboardMapActions[shortcut]) {
        var _action = keyboardMapActions[shortcut];
        this.execute(_action);
      }
      shortcut = '';
    },
    onCanvasLeave: function onCanvasLeave() {
      this.graph.set('focusGraphWrapper', false);
    },
    onCanvasFocus: function onCanvasFocus() {
      this.graph.set('focusGraphWrapper', true);
    },
    onSelected: function onSelected(_ref) {
      var selectedItems = _ref.selectedItems;
      this.selectedItems = selectedItems;
    },
    execute: function execute(action) {
      var methodName = "execute".concat(capitalize(action));
      console.log('execute: ', methodName);
      var method = this[methodName];
      if (method && typeof method === 'function') {
        method.call(this);
      }
      this.graph.emit('afterCommandExecute');
    },
    executeDelete: function executeDelete() {
      var _this$selectedItems;
      var graph = this.graph;
      if ((_this$selectedItems = this.selectedItems) !== null && _this$selectedItems !== void 0 && _this$selectedItems.nodes.length) {
        this.selectedItems.nodes.forEach(function (model) {
          graph.emit('beforeDeleteItem', model.id);
          graph.remove(model.id, false);
        });
        graph.pushStack('delete', {
          before: this.selectedItems,
          after: {}
        }, 'undo');
      } else {
        var item = graph.get('selectedItem');
        if (item) {
          graph.emit('beforeDeleteItem', item);
          graph.remove(item);
          graph.set('selectedItem', null);
          graph.emit('afterItemSelected', null);
        }
      }
    },
    executeCopy: function executeCopy() {
      var _this$selectedItems2;
      var graph = this.graph;
      if ((_this$selectedItems2 = this.selectedItems) !== null && _this$selectedItems2 !== void 0 && _this$selectedItems2.nodes.length) {
        this.clipboard = this.selectedItems;
      } else {
        var id = graph.get('selectedItem');
        var item = graph.findById(id);
        if (item.get('type') === 'node') {
          this.clipboard = item;
        } else {
          this.clipboard = null;
        }
      }
    },
    executePaste: function executePaste() {
      var _this = this;
      if (this.clipboard) {
        var beforeAddNode = this.graph.get('beforeAddNode');
        var beforeAddEdge = this.graph.get('beforeAddEdge');
        if (Array.isArray(this.clipboard.nodes)) {
          var _this$graph2;
          var selectedItems = this.clipboard;
          var copyNodeMap = {};
          var addNodes = selectedItems.nodes.map(function (model) {
            var newId = uniqIdByDate(model.type);
            copyNodeMap[model.id] = newId;
            var addModel = _extends({}, model, {
              id: newId,
              x: model.x + 20,
              y: model.y + 20
            });
            beforeAddNode && beforeAddNode(_this.graph, addModel);
            _this.graph.add('node', addModel, false);
            return addModel;
          });
          var addEdges = selectedItems.edges.map(function (model) {
            var newSourceId = copyNodeMap[model.source];
            var newTargetId = copyNodeMap[model.target];
            if (newSourceId && newTargetId) {
              var _this$graph;
              var addModel = _extends({}, model, {
                id: uniqIdByDate(model.type),
                source: newSourceId,
                target: newTargetId,
                x: model.x + 20,
                y: model.y + 20
              });
              beforeAddEdge && beforeAddEdge(_this.graph, _this.graph.findById(newSourceId), _this.graph.findById(newTargetId), addModel);
              (_this$graph = _this.graph) === null || _this$graph === void 0 ? void 0 : _this$graph.add('edge', clone(addModel), false);
              return addModel;
            }
            return null;
          }).filter(Boolean);
          (_this$graph2 = this.graph) === null || _this$graph2 === void 0 ? void 0 : _this$graph2.pushStack('add', {
            after: {
              nodes: addNodes,
              edges: addEdges
            }
          }, 'undo');
          clearSelected(this.graph);
          setSelected(this.graph, addNodes.concat(addEdges));
        } else {
          var _this$graph3;
          var model = this.clipboard.getModel();
          var addModel = _extends({}, model, {
            id: uniqIdByDate(model.type),
            x: model.x + 20,
            y: model.y + 20
          });
          beforeAddNode && beforeAddNode(this.graph, addModel);
          (_this$graph3 = this.graph) === null || _this$graph3 === void 0 ? void 0 : _this$graph3.add('node', addModel);
        }
      }
      this.clipboard = null;
    },
    executeZoomIn: function executeZoomIn() {
      var graph = this.graph;
      var zoom = graph.getZoom();
      graph.zoomTo(zoom + 0.1);
    },
    executeZoomOut: function executeZoomOut() {
      var graph = this.graph;
      var zoom = graph.getZoom();
      graph.zoomTo(zoom - 0.1);
    },
    executeUndo: function executeUndo() {
      var graph = this.graph;
      var undoStack = graph.getUndoStack();
      if (!undoStack || undoStack.length === 0) return;
      var currentData = undoStack.pop();
      if (!currentData) return;
      var action = currentData.action;
      graph.pushStack(action, clone(currentData.data), 'redo');
      var beforeData = currentData.data.before;
      if (action === 'add') {
        beforeData = currentData.data.after;
      }
      if (!beforeData) return;
      switch (action) {
        case 'render':
        case 'update':
          Object.keys(beforeData).forEach(function (key) {
            var arr = beforeData[key];
            if (!arr || arr.length === 0) return;
            arr.forEach(function (model) {
              graph.updateItem(model.id, model, false);
            });
          });
          break;
        case 'changedata':
          graph.changeData(beforeData, false);
          break;
        case 'delete':
          Object.keys(beforeData).forEach(function (key) {
            var arr = beforeData[key];
            if (!arr || arr.length === 0) return;
            arr.forEach(function (model) {
              graph.addItem(key === 'edges' ? 'edge' : 'node', model, false);
            });
          });
          break;
        case 'add':
          Object.keys(beforeData).forEach(function (key) {
            var arr = beforeData[key];
            if (!arr || arr.length === 0) return;
            arr.forEach(function (model) {
              graph.removeItem(model.id, false);
            });
          });
          break;
        default:
      }
    },
    executeRedo: function executeRedo() {
      var graph = this.graph;
      var redoStack = graph.getRedoStack();
      if (!redoStack || redoStack.length === 0) return;
      var currentData = redoStack.pop();
      if (!currentData) return;

      // console.log(currentData);
      var action = currentData.action;
      var afterData = currentData.data.after;
      graph.pushStack(action, clone(currentData.data));
      if (action === 'delete') {
        afterData = currentData.data.before;
      }
      if (!afterData) return;
      switch (action) {
        case 'render':
        case 'update':
          Object.keys(afterData).forEach(function (key) {
            var arr = afterData[key];
            if (!arr || arr.length === 0) return;
            arr.forEach(function (model) {
              graph.updateItem(model.id, model, false);
            });
          });
          break;
        case 'changedata':
          graph.changeData(afterData, false);
          break;
        case 'delete':
          if (afterData.edges) {
            afterData.edges.forEach(function (model) {
              graph.removeItem(model.id, false);
            });
          }
          if (afterData.nodes) {
            afterData.nodes.forEach(function (model) {
              graph.removeItem(model.id, false);
            });
          }
          break;
        case 'add':
          Object.keys(afterData).forEach(function (key) {
            var arr = afterData[key];
            if (!arr || arr.length === 0) return;
            arr.forEach(function (model) {
              graph.addItem(key === 'edges' ? 'edge' : 'node', model, false);
            });
          });
          break;
      }
    },
    executeAutoFit: function executeAutoFit() {
      this.graph.fitView(5);
    },
    executeResetZoom: function executeResetZoom() {
      this.graph.zoomTo(1);
    }
  });
}