import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import { deepMix, each, wrapBehavior } from '@antv/util';
var PluginBase = /*#__PURE__*/function () {
  /**
   * 插件基类的构造函数
   * @param cfgs 插件的配置项
   */
  function PluginBase(cfgs) {
    _classCallCheck(this, PluginBase);
    this._events = void 0;
    this._cfgs = void 0;
    this.destroyed = void 0;
    this._cfgs = deepMix(this.getDefaultCfgs(), cfgs);
    this._events = {};
    this.destroyed = false;
  }

  /**
   * 获取默认的插件配置
   */
  _createClass(PluginBase, [{
    key: "getDefaultCfgs",
    value: function getDefaultCfgs() {
      return {};
    }

    /**
     * 初始化插件
     * @param graph IGraph 实例
     */
  }, {
    key: "initPlugin",
    value: function initPlugin(graph) {
      var self = this;
      self.set('graph', graph);
      var events = self.getEvents();
      var bindEvents = {};
      each(events, function (v, k) {
        var event = wrapBehavior(self, v);
        bindEvents[k] = event;
        graph.on(k, event);
      });
      this._events = bindEvents;
      this.init();
    }

    /**
     * 初始化方法，供子类实现
     */
  }, {
    key: "getEvents",
    value:
    /**
     * 获取插件中的事件和事件处理方法，供子类实现
     */
    function getEvents() {
      return {};
    }

    /**
     * 获取配置项中的某个值
     * @param key 键值
     */
  }, {
    key: "get",
    value: function get(key) {
      var _this$_cfgs;
      return (_this$_cfgs = this._cfgs) === null || _this$_cfgs === void 0 ? void 0 : _this$_cfgs[key];
    }

    /**
     * 将指定的值存储到 cfgs 中
     * @param key 键值
     * @param val 设置的值
     */
  }, {
    key: "set",
    value: function set(key, val) {
      this._cfgs[key] = val;
    }

    /**
     * 销毁方法，供子类复写
     */
  }, {
    key: "destroy",
    value: function destroy() {}

    /**
     * 销毁插件
     */
  }, {
    key: "destroyPlugin",
    value: function destroyPlugin() {
      this.destroy();
      var graph = this.get('graph');
      var events = this._events;
      each(events, function (v, k) {
        graph.off(k, v);
      });
      this._events = null;
      this._cfgs = null;
      this.destroyed = true;
    }
  }]);
  return PluginBase;
}();
export { PluginBase as default };