import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _extends from "@babel/runtime/helpers/esm/extends";
import { getShapeName } from './clazz';
var PointIndex = ['TopCenter', 'RightMiddle', 'BottomCenter', 'LeftMiddle'];
export var FlowStatusColors = {
  '0': {
    // 未开始
    stroke: '#dedede',
    fill: '#dedede'
  },
  '1': {
    // 进行中
    stroke: '#feaf26',
    fill: '#fffae6'
  },
  '2': {
    // 已完成
    stroke: '#7cbd1b',
    fill: '#f5ffe6'
  }
};
export var defaultData = {
  nodes: [{
    clazz: 'START',
    size: [50, 50],
    label: '开始',
    shape: 'start-node',
    type: 'start-node',
    x: 234,
    y: 50,
    id: 'startEvent-1586419748018',
    defCategory: 'START',
    defKey: 'startEvent-1586419748018',
    defName: '开始',
    userChooseType: '1',
    userChooseMethod: '1',
    roleIds: [],
    groupId: 1572508657824,
    nodeIdIndex: 0
  }, {
    clazz: 'NORMAL',
    size: [90, 40],
    label: '正常任务',
    shape: 'normal-node',
    type: 'normal-node',
    x: 234,
    y: 150,
    id: 'process-node-1586419956248',
    defCategory: 'NORMAL',
    defKey: 'process-node-1586419956248',
    defName: '正常任务',
    userChooseType: '1',
    userChooseMethod: '1',
    roleIds: [{}],
    isProcFirst: true,
    groupId: 1572508657824,
    nodeIdIndex: 1
  }],
  edges: [{
    clazz: 'flow',
    source: 'startEvent-1586419748018',
    target: 'process-node-1586419956248',
    sourceTaskDefKey: 'startEvent-1586419748018',
    tarTaskDefKey: 'process-node-1586419956248',
    sourceAnchor: 2,
    id: 'Flow-0',
    targetAnchor: 0,
    shape: 'flow-polyline-round',
    type: 'flow-polyline-round',
    flowName: '提交',
    startPoint: {
      x: 239.5,
      y: 190.5,
      index: 2
    },
    endPoint: {
      x: 240,
      y: 272.5,
      index: 0
    },
    lineIdIndex: 0
  }]
};
export function getNodeSize(clazz, node) {
  var width = Math.max(node.defName.length * 16, 90);
  return {
    START: [50, 50],
    END: [50, 50],
    NORMAL: [width, 40],
    PARALLEL_SIGN: [width, 40],
    SERVICE: [width, 40]
  }[clazz] || [];
}
export function getPointPosition(type, node) {
  switch (type) {
    case 'TopCenter':
      return {
        x: node.x + node.size[0] / 2,
        y: node.y,
        index: 0
      };
    case 'RightMiddle':
      return {
        x: node.x + node.size[0],
        y: node.y + node.size[1] / 2,
        index: 1
      };
    case 'BottomCenter':
      return {
        x: node.x + node.size[0],
        y: node.y + node.size[1],
        index: 2
      };
    default:
      return {
        x: node.x,
        y: node.y + node.size[1] / 2,
        index: 3
      };
  }
}
export function dealFlowData(data, flowTypes, usedFlowTypes, dynamicTasks, setDynamicData) {
  if (!data) return _extends({}, defaultData);
  console.log('flowData：', data);
  var nodes = [];
  var edges = [];
  var flowTypeMap = new Map(flowTypes.map(function (item) {
    return [item.id, {
      name: item.val,
      color: item.extVal
    }];
  }));
  var useMap = {};
  var nodeMap = {};
  data.tasks && Object.entries(data.tasks).forEach(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
      key = _ref2[0],
      sourceNode = _ref2[1];
    var node = JSON.parse(JSON.stringify(sourceNode));
    if (node.__groupId) {
      node.groupId = node.__groupId;
      delete node.__groupId;
    }
    if (node.__parentGroupId) {
      node.parentGroupId = node.__parentGroupId;
      delete node.__parentGroupId;
    }
    delete node.labelCfg;
    node.clazz = node.defCategory;
    node.label = node.defName;
    node.id = node.nodeKey;
    node.type = getShapeName(node.clazz);
    if (!node.size) {
      node.size = getNodeSize(node.clazz, node);
    }
    if (!node.x || !node.y) {
      node.x = node.col * 200;
      node.y = node.row * 100 + 30;
    }
    if (node.autoTime && typeof node.autoTime === "number") {
      node.autoTime = node.autoTime / 3600000;
    }
    if (node.userChooseType) {
      node.userChooseType = node.userChooseType + '';
    }
    if (node.userChooseMethod) {
      node.userChooseMethod = node.userChooseMethod + '';
    }

    // 设置动态数据
    if (dynamicTasks) {
      var dynamicNode = dynamicTasks[key];
      console.log('dynamicTask： ', dynamicNode);
      if (dynamicNode && typeof dynamicNode.mark !== "undefined") {
        node.style = FlowStatusColors[dynamicNode.mark];
        node.assigneeNames = dynamicNode.assigneeNames;
        setDynamicData && setDynamicData(node, dynamicNode);
      }
    }
    nodeMap[key] = node;
    nodes.push(node);
  });
  data.flows && Object.entries(data.flows).forEach(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
      sourceEdge = _ref4[1];
    var edge = JSON.parse(JSON.stringify(sourceEdge));
    edge.source = edge.sourceTaskNodeKey;
    edge.sourceType = nodeMap[edge.sourceTaskDefKey].defCategory;
    edge.target = edge.tarTaskNodeKey;
    edge.targetType = nodeMap[edge.tarTaskDefKey].defCategory;
    edge.clazz = 'flow';
    edge.type = 'flow';
    if (PointIndex.includes(edge.sourceAnchor)) {
      edge.startPoint = getPointPosition(edge.sourceAnchor, nodeMap[edge.sourceTaskDefKey]);
      edge.sourceAnchor = edge.startPoint.index;
    }
    if (PointIndex.includes(edge.targetAnchor)) {
      edge.endPoint = getPointPosition(edge.targetAnchor, nodeMap[edge.tarTaskDefKey]);
      edge.targetAnchor = edge.endPoint.index;
    }
    var usedFlowType = flowTypeMap.get(String(edge.flowType));
    if (usedFlowType) {
      edge.flowType = String(edge.flowType);
      edge.color = usedFlowType.color;
      if (!useMap[edge.flowType]) {
        useMap[edge.flowType] = true;
        usedFlowTypes.push(flowTypeMap.get(edge.flowType));
      }
    }
    if (edge.autoTime && typeof edge.autoTime === "number") {
      edge.autoTime = edge.autoTime / 3600000;
    }
    edges.push(edge);
  });
  if (nodes.length > 0 || edges.length > 0) {
    return {
      nodes: nodes,
      edges: edges
    };
  }
  return defaultData;
}