import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _construct from "@babel/runtime/helpers/esm/construct";
import DayJs from 'dayjs';
import { isEmpty, isNumber, isString, isUndefined } from '../types';
import { dfsTravelNode } from "./form-util";
var varRegexp = /{[a-zA-Z_$][0-9a-zA-Z_$]*}/;
var countDateDiffReg = /countDateDiff\({[a-zA-Z_$][0-9a-zA-Z_$]*},\s*'(day|month|year)'\)/;
var dateFormatReg = /dateFormat\({[a-zA-Z_$][0-9a-zA-Z_$]*},'([yY]{4}[-/]?MM[-/]?[dD]{2}\s*[hHms:]*)'\)/;
function bracketsMatch(str) {
  var len = str.length;
  var pairs = new Map([[')', '('], [']', '['], ['}', '{']]);
  var stk = [];
  var leftBrackets = Array.from(pairs.values());
  for (var i = 0; i < len; i++) {
    var ch = str[i];
    if (leftBrackets.includes(ch)) {
      stk.push(ch);
    } else if (pairs.has(ch)) {
      if (!stk.length || stk[stk.length - 1] !== pairs.get(ch)) return false;
      stk.pop();
    }
  }
  return !stk.length;
}
function executeFunction(expr, scopes) {
  var args = Object.keys(scopes);
  var fn = _construct(Function, args.concat(['return ' + expr]));
  return fn.apply(void 0, _toConsumableArray(args.map(function (key) {
    return scopes[key];
  })));
}
var Expression = {
  /**
   * 根据字段key,找到对应的字段对象
   * @param formList {Array}
   * @param fieldKey {string}
   * @return {*}
   */
  findFieldByKey: function findFieldByKey(formList, fieldKey) {
    var target;
    dfsTravelNode(formList, function (field) {
      if (field.fieldKey === fieldKey) {
        target = field;
        return false;
      }
      return true;
    });
    return target;
  },
  /**
   * 校验表达式字段是否在表单字段中
   * @param exprObj {Object}
   * @param formList {Array}
   */
  validateExprFieldInFormList: function validateExprFieldInFormList(exprObj, formList) {
    var _this = this;
    var formComputeExprs = exprObj.formComputeExprs,
      formValidateExprs = exprObj.formValidateExprs;
    var traverseExprList = function traverseExprList(list) {
      var keyMap = {};
      if (Array.isArray(list)) {
        list.forEach(function (item) {
          var expr = item.expr;
          var result = varRegexp.exec(expr);
          while (result) {
            var fieldKey = result[0].substring(1, result[0].length - 1);
            if (!keyMap[fieldKey]) {
              var obj = _this.findFieldByKey(formList, fieldKey);
              if (!obj) {
                throw Error("\u8868\u8FBE\u5F0F\u4E2D\u5B58\u5728\u5B57\u6BB5: ".concat(fieldKey, "\uFF0C\u4F46\u5728\u8868\u5355\u5B57\u6BB5\u4E2D\u627E\u4E0D\u5230\uFF0C\u8BF7\u68C0\u67E5"));
              } else {
                keyMap[fieldKey] = 1;
              }
            }
            expr = expr.replace(new RegExp(result[0], 'g'), fieldKey);
            result = varRegexp.exec(expr);
          }
        });
      }
    };
    try {
      traverseExprList(formValidateExprs.exprList);
    } catch (e) {
      return new Error("\u5B57\u6BB5\u6821\u9A8C".concat(e));
    }
    try {
      traverseExprList(formComputeExprs.exprList);
    } catch (e) {
      return new Error("\u5B57\u6BB5\u8BA1\u7B97".concat(e));
    }
    return true;
  },
  /**
   * 验证赋值表达式的合法性
   * 例子：{a} + {b} = {c}
   * 规则：
   *     1、等号只能有一个
   *     2、等号右侧只能有一个字段
   *     3、括号匹配
   *     4、符合运算规则
   * @param expStr
   * @returns
   */
  validateExpression: function validateExpression(expStr) {
    if (!expStr) {
      return new Error('不能为空');
    }
    var _Expression$splitComp = Expression.splitComputeExpr(expStr.replace(/\s+/g, ''), false),
      _Expression$splitComp2 = _slicedToArray(_Expression$splitComp, 2),
      computeStr = _Expression$splitComp2[0],
      resultKey = _Expression$splitComp2[1];

    // 等号右侧只能有一个字段
    if (!resultKey) {
      return new Error('等号右侧有且只能有一个字段, 并且字段名用{}包裹');
    }
    if (!varRegexp.test(resultKey)) {
      return new Error('等号右侧有且只能有一个字段, 并且字段名用{}包裹');
    }
    var right = resultKey.split(varRegexp);
    if (right[0] !== right[1]) {
      return new Error('等号右侧有且只能有一个字段, 并且字段名用{}包裹');
    }

    // 括号匹配
    if (!bracketsMatch(computeStr)) {
      return new Error('括号不匹配');
    }

    // 检查countDateDiff表达式是否正确
    if (/countDateDiff/.test(computeStr)) {
      var countDateDiff = computeStr.match(countDateDiffReg);
      if (countDateDiff) {
        computeStr = computeStr.replace(countDateDiff[0], '0');
      } else {
        return new Error('countDateDiff() 方法，用法有误，请检查');
      }
    }
    if (/dateFormat/.test(computeStr)) {
      var dateFormat = computeStr.match(dateFormatReg);
      console.log(dateFormat);
      if (dateFormat) {
        computeStr = computeStr.replace(dateFormat[0], '0');
      } else {
        return new Error('dateFormat() 方法，用法有误，请检查');
      }
    }

    // 符合运算规则, 使用eval测试
    var re = varRegexp.exec(computeStr);
    var result = '';
    while (re) {
      // console.log(re);
      result = re.input.replace(new RegExp(re[0]), '0');
      re = varRegexp.exec(result);
    }
    try {
      var fn = new Function('return ' + result);
      fn();
    } catch (e) {
      return Error('表达式有误');
    }
    return true;
  },
  /**
   * 校验逻辑表达式，逻辑运算符： >、< 、>=、<=、=== 、==、&&、||、!
   * 规则：
   *     1、括号匹配
   *     2、符合语法规则
   * @param logicExpr
   * @param {Array} formList
   */
  validateLogicExpr: function validateLogicExpr(logicExpr, formList) {
    // 1、括号匹配
    if (!bracketsMatch(logicExpr)) {
      return new Error('括号不匹配');
    }

    // 2、符合语法
    var re = varRegexp.exec(logicExpr);
    var result = '';
    while (re) {
      // console.log(re);
      var fieldKey = re[0].substring(1, re[0].length - 1);
      var field = this.findFieldByKey(formList, fieldKey);
      if (field) {
        result = re.input.replace(new RegExp(re[0]), '0');
        re = varRegexp.exec(result);
      } else {
        return Error("\u8868\u8FBE\u5F0F\u6709\u8BEF: ".concat(fieldKey, "\u5B57\u6BB5\u4E0D\u5B58\u5728"));
      }
    }
    // console.log(result);
    try {
      var fn = new Function('return ' + result);
      fn();
    } catch (e) {
      return Error('表达式有误');
    }
    return true;
  },
  /**
   * 编译表达式, 替换表达式中的变量
   * @param  {string} expr
   * @param  {string} scopeName
   * @return {String} result
   */
  compileExprStr: function compileExprStr(expr, scopeName) {
    var re = varRegexp.exec(expr);
    var result = expr;
    while (re) {
      // console.log(re);
      var key = re[0].substring(1, re[0].length - 1);
      result = re.input.replace(new RegExp(re[0], 'g'), "".concat(scopeName, ".").concat(key));
      re = varRegexp.exec(result);
    }
    return result;
  },
  /**
   * 编译表达式, 判断变量key归属， 替换表达式中的变量
   * @param  {string} expr
   * @param  {object} scopes
   * @return {String} result
   */
  compileExprStrJudgeScope: function compileExprStrJudgeScope(expr, scopes) {
    var formData = scopes.formData,
      params = scopes.params,
      procDef = scopes.procDef,
      processInfo = scopes.processInfo;
    var re = varRegexp.exec(expr);
    // 表达式中没有变量替换，直接返回
    if (!re) {
      return executeFunction(expr, scopes);
    }
    var result = expr;
    while (re) {
      var key = re[0].substring(1, re[0].length - 1);
      var scope = '';
      if (formData && key in formData) {
        scope = 'formData';
      } else if (params && key in params) {
        scope = 'params';
      } else if (procDef && key in procDef) {
        scope = 'procDef';
      } else if (processInfo && key in processInfo) {
        scope = 'processInfo';
      }
      if (!scope) {
        console.warn("\u8868\u8FBE\u5F0F\u5F02\u5E38\uFF1A".concat(key, " \u5B57\u6BB5\u4E0D\u5B58\u5728"));
        scope = 'params';
      }
      result = re.input.replace(new RegExp(re[0], 'g'), "".concat(scope, ".").concat(key));
      re = varRegexp.exec(result);
    }
    return executeFunction(result, scopes);
  },
  /**
   * 执行逻辑表达式
   * 问题点：
   *     1、如何划分计算子式？
   *          按照逻辑运算符，分割表达式
   *     2、有小括号又如何处理？
   *          层层去除小括号
   *
   * @param logicExpr {string}
   * @param formData {Object}
   * @param accuracy 表达式中存在计算，计算精度
   * @return result {boolean | Error}
   */
  executeLogicExpr: function executeLogicExpr(logicExpr, formData, accuracy) {
    var leftBracket = [];
    var exprStack = [];
    var splitReg = /[><!]|>=|<=|===|==|&&|&#124;&#124;/;
    var isCompute = /[+\-*/]|Math/;

    // 按照括号分组
    var tmpLogicExpr = logicExpr;
    for (var i = 0; i < logicExpr.length; i++) {
      if (logicExpr[i] === '(') {
        leftBracket.push(i);
        continue;
      }
      if (logicExpr[i] === ')' && leftBracket.length !== 0) {
        var start = leftBracket.pop();
        var tmp = logicExpr.substring((start || 0) + 1, i);
        if (isCompute.test(tmp)) {
          // 暂存括号包含的分组结果
          exprStack.push(tmp);
        }
      }
    }

    // 替换子式
    var replaceSub = function replaceSub(expr) {
      var compiledExpr = expr;
      var subLogicExprs = expr.split(splitReg);
      var result;
      subLogicExprs.forEach(function (sub) {
        if (isCompute.test(sub)) {
          try {
            var _result = executeFunction(Expression.compileExprStr(sub, 'formData'), {
              formData: formData
            });
            var _tmp = _result.toFixed(accuracy);
            compiledExpr = compiledExpr.replace(sub, _tmp);
          } catch (e) {
            console.error(e);
          }
        } else if (varRegexp.test(sub)) {
          var _tmp2 = Expression.compileExprStr(sub, 'formData');
          compiledExpr = compiledExpr.replace(sub, _tmp2);
        }
      });
      // 执行表达式
      if (splitReg.test(compiledExpr) || isCompute.test(compiledExpr)) {
        result = executeFunction(Expression.compileExprStr(compiledExpr, 'formData'), {
          formData: formData
        });
      } else {
        result = compiledExpr;
      }
      return result;
    };
    if (exprStack.length === 0) {
      return replaceSub(tmpLogicExpr);
    }
    for (var _i = 0; _i < exprStack.length; _i++) {
      // 按逻辑运算符分组，计算
      var result = replaceSub(exprStack[_i]);

      // 当前表达式包含在下一个表达式中
      if (_i !== exprStack.length - 1 && exprStack[_i + 1].match(exprStack[_i])) {
        exprStack[_i + 1] = exprStack[_i + 1].replace(exprStack[_i], result);
      }
      tmpLogicExpr = tmpLogicExpr.replace(exprStack[_i], result);
    }

    // console.log(logicExpr);
    try {
      return executeFunction(tmpLogicExpr, {
        formData: formData
      });
    } catch (e) {
      console.error(e);
    }
  },
  /**
   * 执行条件表达式
   * @param expr
   * @param scopes  可以的scope： formList, params, procDef, processInfo
   */
  executeConditionExpr: function executeConditionExpr(expr, scopes) {
    if (!expr) return false;
    try {
      return Expression.compileExprStrJudgeScope(expr, scopes);
    } catch (e) {
      console.log("\u8868\u8FBE\u5F0F\u9519\u8BEF\uFF1A".concat(expr));
      console.warn(e);
      return false;
    }
  },
  /**
   * 执行表单的逻辑表达式， 如果出错，则返回错误信息
   * @param formValidateExprs
   * @param formData
   * @return {string}
   */
  executeValidateExpr: function executeValidateExpr(formValidateExprs, formData) {
    var errorPrompt = '';
    if (!formValidateExprs) return errorPrompt;
    for (var i = 0, len = formValidateExprs.exprList.length; i < len; i++) {
      var item = formValidateExprs.exprList[i];
      var result = Expression.executeLogicExpr(item.expr, formData, formValidateExprs.accuracy);
      if (!result) {
        return item.errorPrompt;
      }
    }
    return errorPrompt;
  },
  /**
   * 计算时间差值
   * @param field
   * @param type
   * @returns {number}
   */
  countDateDiff: function countDateDiff(field, type) {
    var diff = 0;
    if (field && field[0] && field[1]) {
      var startTime = DayJs(new Date(field[0]));
      var endTime = DayJs(new Date(field[1]));
      switch (type) {
        case 'day':
          return endTime.diff(startTime, 'days');
        case 'month':
          return endTime.diff(startTime, 'months');
        case 'year':
          return endTime.diff(startTime, 'years');
        default:
          break;
      }
    }
    return diff;
  },
  /**
   * 计算策略
   * @param computeStr {string} 计算表达式
   * @param formData {object}
   */
  computeStrategy: function computeStrategy(computeStr, formData) {
    // 计算时间间隔
    var countDateDiff = computeStr.match(countDateDiffReg);
    if (countDateDiff) {
      var subStr = countDateDiff[0];
      var type = countDateDiff[1];
      var matchedVariable = subStr.match(varRegexp);
      if (matchedVariable) {
        var key = matchedVariable[0];
        key = key.substring(1, key.length - 1);
        var objField = formData[key];
        var result = Expression.countDateDiff(objField, type);
        computeStr = computeStr.replace(subStr, result + '');
      }
    }
    var dateFormat = computeStr.match(dateFormatReg);
    if (dateFormat) {
      var _subStr = dateFormat[0];
      var formatStr = dateFormat[1];
      var _matchedVariable = _subStr.match(varRegexp);
      if (_matchedVariable) {
        var _key = _matchedVariable[0];
        _key = _key.substring(1, _key.length - 1);
        var _objField = formData[_key];
        var _result2 = DayJs(new Date(Number(_objField))).format(formatStr);
        computeStr = computeStr.replace(_subStr, _result2);
      }
    }
    return computeStr;
  },
  /**
   * 执行表达式，并将计算结果返回
   * @param exprObj {object} 计算表达式
   * @param formData {object} 表单对象
   * @param scopes {object} 字段作用域
   * @returns {*}
   */
  executeComputeExpr: function executeComputeExpr(exprObj, formData, scopes) {
    var computeExpr = exprObj.computeExpr,
      resultKey = exprObj.resultKey,
      fullExpr = exprObj.fullExpr,
      accuracy = exprObj.accuracy;
    accuracy = !isUndefined(accuracy) ? accuracy : 2;
    var computeStr = Expression.computeStrategy(computeExpr, formData);
    // console.log(computeExpr)
    try {
      if (isString(computeStr)) {
        var result = Expression.compileExprStrJudgeScope(computeStr, scopes);
        formData[resultKey] = isNumber(result) ? result.toFixed(accuracy) : result;
      } else {
        formData[resultKey] = computeStr;
      }
    } catch (e) {
      console.log(e);
      alert("\u8868\u8FBE\u5F0F:".concat(fullExpr, "\u6709\u8BEF"));
      return formData;
    }
    return formData;
  },
  /**
   * 拆分计算表达式
   * @param exprStr
   * @param trimWrapper
   * @return {(string|string)[]}
   */
  splitComputeExpr: function splitComputeExpr(exprStr) {
    var trimWrapper = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var lastEqualSignIndex = exprStr.lastIndexOf('=');
    var computeExpr = exprStr.slice(0, lastEqualSignIndex).trim();
    var resultStr = exprStr.slice(lastEqualSignIndex + 1).trim();
    var resultKey = trimWrapper ? resultStr.substring(1, resultStr.length - 1) : resultStr;
    return [computeExpr, resultKey];
  },
  /**
   * 判断计算表达式里面的变量都有值
   * @param computeExpr
   * @param scopes
   */
  judgeComputeExprVariableHasValue: function judgeComputeExprVariableHasValue(computeExpr, scopes) {
    var re = varRegexp.exec(computeExpr);
    // 表达式中没有变量，直接返回
    if (!re) {
      return true;
    }
    var formData = scopes.formData,
      procDef = scopes.procDef,
      params = scopes.params,
      processInfo = scopes.processInfo;
    var hasValue = function hasValue(obj, key) {
      return obj ? !isEmpty(obj[key]) : false;
    };
    var pass = false;
    var resultStr = computeExpr;
    while (re) {
      var key = re[0].substring(1, re[0].length - 1);
      if (hasValue(formData, key) || hasValue(procDef, key) || hasValue(params, key) || hasValue(processInfo, key)) {
        resultStr = re.input.replace(new RegExp(re[0], 'g'), key);
        re = varRegexp.exec(resultStr);
        pass = true;
      } else {
        return false;
      }
    }
    return pass;
  },
  /**
   * 根据字段key, 排序需要执行的计算表达式
   * @param exprList
   * @param fieldKey
   * @returns {Array}
   */
  sortExprByFiledKey: function sortExprByFiledKey(exprList, fieldKey) {
    var resultKeys = [];
    var executeSorts = [];
    exprList.forEach(function (item) {
      var _Expression$splitComp3 = Expression.splitComputeExpr(item.expr),
        _Expression$splitComp4 = _slicedToArray(_Expression$splitComp3, 2),
        computeExpr = _Expression$splitComp4[0],
        resultKey = _Expression$splitComp4[1];
      if (computeExpr.match(fieldKey)) {
        resultKeys.push(resultKey);
        executeSorts.push({
          fullExpr: item.expr,
          computeExpr: computeExpr,
          resultKey: resultKey
        });
      }
    });
    resultKeys.forEach(function (key) {
      executeSorts = executeSorts.concat(Expression.sortExprByFiledKey(exprList, key));
    });
    return executeSorts;
  },
  /**
   * 批量执行表单的计算表达式
   * @param formComputeExprs
   * @param fieldKey
   * @param formData
   * @param scopes
   */
  bachExecuteComputeExpr: function bachExecuteComputeExpr(formComputeExprs, fieldKey, formData, scopes) {
    if (!formComputeExprs || !formComputeExprs.exprList) return formData;
    // 判断表达式的执行顺序
    var exprList = Expression.sortExprByFiledKey(formComputeExprs.exprList, fieldKey);
    exprList.forEach(function (item) {
      // 判断表达式里面的变量，是否都有值
      var pass = Expression.judgeComputeExprVariableHasValue(item.computeExpr, scopes);
      if (pass) {
        formData = Expression.executeComputeExpr(item, formData, scopes);
      }
    });
    return formData;
  },
  /**
   * 过滤表达式中的变量
   * @param expr
   */
  filterExpressionVariable: function filterExpressionVariable(expr) {
    var fields = [];
    if (!expr) return fields;
    var re = varRegexp.exec(expr);
    // 表达式中没有变量，直接返回
    if (!re) {
      return fields;
    }
    while (re) {
      var key = re[0].substring(1, re[0].length - 1);
      fields.push(key);
      expr = re.input.replace(new RegExp(re[0], 'g'), key);
      re = varRegexp.exec(expr);
    }
    return fields;
  }
};
export default Expression;