// 流程 任务操作
import { FILE_BUSI_TYPE, USER_CHOOSE_METHOD } from "../constants";
var nextTaskParamListKeys = ["abnormalProc",
// 是否是异常结束流程
"canSendEMail", "canSendMsg", "canSendWeChat", "category",
// 任务类型
"groupId", "mulInstKey", "nextTaskAssigneeParmList", "parentGroupId", "taskDefKey", "specialSubproc", "withoutViewSubmit"];
var taskOperateDetailKeys = ["canAppEdit",
// app端操作
"canEditPage",
// 编辑页面
"canDeleteMyAtt",
// 删除我上传的附件
"canDeleteOtherAtt", "canDownloadAtt", "canReadAtt", "canUpdateMyAtt", "canUpdateOtherAtt", "canUploadAtt", "passRead", "haveAddSign", "haveCounterSign", "haveDelegate", "haveOpinionText",
// 是否显示处理意见框
"necOpinionText",
// 处理意见必填

"needChooseUser",
//是否需要选择处理人
"userChooseMethod" // 选人方式
];

var ProcessUtil = {
  /**
   * 从user获取申请人信息
   * @param user
   */
  getApplicantInfo: function getApplicantInfo(user) {
    // 主岗可能不是第一个
    var a = [],
      b = [];
    user.userJobs.forEach(function (item) {
      if (item.isMainjob === 'Y') {
        a.push(item);
      } else {
        b.push(item);
      }
    });
    user.userJobs = a.concat(b);
    // console.log(user.userJobs);
    return {
      compId: user.userJobs && user.userJobs[0] ? user.userJobs[0].companyId || '' : user.userGroupIdPath ? user.userGroupIdPath.split(',')[1] : '',
      compName: user.userJobs && user.userJobs[0] ? user.userJobs[0].companyName : user.userRootGroupName,
      compCode: user.userJobs && user.userJobs[0] ? user.userJobs[0].companyCode : '',
      mobile: user.mobile || '',
      orgId: user.userJobs && user.userJobs.length > 0 ? user.userJobs[0].orgId : user.userGroupId,
      orgName: user.userJobs && user.userJobs.length > 0 ? user.userJobs[0].orgName : user.userGroupName,
      orgCode: user.userJobs && user.userJobs.length > 0 ? user.userJobs[0].orgCode : '',
      postId: user.userJobs && user.userJobs.length > 0 ? user.userJobs[0].postId : '',
      postName: user.userJobs && user.userJobs.length > 0 ? user.userJobs[0].postName : '',
      postCode: user.userJobs && user.userJobs.length > 0 ? user.userJobs[0].postCode : '',
      userId: user.id,
      userCode: user.userCode,
      email: user.email,
      userName: user.userName
    };
  },
  /**
   * 从流程历史中取到申请人信息
   * @param data
   * @return  {Object}
   */
  setApplicantInfoByProcHis: function setApplicantInfoByProcHis(data) {
    return {
      compId: data.procStartStaffCompanyId,
      compName: data.procStartStaffCompanyName,
      mobile: data.procStartStaffPhone,
      orgId: data.procStartStaffOrgId,
      orgName: data.procStartStaffOrgName,
      userId: data.procStartStaffId,
      userName: data.procStartStaff
    };
  },
  /**
   * 设置任务操作的详细
   * @param curTask
   * @param defaultValue
   */
  setCurTaskOperateDetail: function setCurTaskOperateDetail(curTask) {
    var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    try {
      var obj = {};
      taskOperateDetailKeys.forEach(function (key) {
        // 附件的下载和预览，当前任务操作不存在时，默认为true
        if (["canDownloadAtt", "canReadAtt"].indexOf(key) > -1) {
          obj[key] = curTask === null || typeof curTask[key] === "undefined" ? true : curTask[key];
        } else {
          obj[key] = curTask ? curTask[key] : defaultValue;
        }
      });
      // console.log('taskDetail', obj);
      return obj;
    } catch (e) {
      console.warn('setCurTaskOperateDetail');
    }
  },
  /**
   * 处理文件
   * @param data
   * @param busiType
   * @return {Array}
   */
  dealFileInfo: function dealFileInfo(data, busiType) {
    var fileList = [];
    if (Array.isArray(data)) {
      var obj = data.find(function (item) {
        return item.busiType === FILE_BUSI_TYPE[busiType];
      });
      if (obj) {
        fileList = obj.fileInfo || [];
      }
    }
    return fileList;
  },
  /**
   * 处理下一步任务列表
   */
  dealTotalNextTaskList: function dealTotalNextTaskList(totalNextTaskList) {
    var nextTaskList = [];
    var group = [],
      sort = [];
    totalNextTaskList = totalNextTaskList || [];
    totalNextTaskList.forEach(function (item) {
      if (!sort.includes(item.operation)) {
        sort.push(item.operation);
      }
      if (!group[item.operation]) {
        group[item.operation] = [item];
      } else {
        group[item.operation].push(item);
      }
    });
    sort.forEach(function (operation) {
      var first = group[operation][0];
      var tmp = {
        operation: operation,
        operationName: first.operationName,
        taskName: first.taskName,
        detail: group[operation]
      };
      if (group[operation].length > 1) {
        tmp.taskName = '条件判断';
      }
      nextTaskList.push(tmp);
    });
    return nextTaskList;
  },
  /**
   *
   * @param nextTaskList
   * @param budgetList
   * @return {Array}
   */
  matchNextTaskListByBudget: function matchNextTaskListByBudget(nextTaskList, budgetList) {
    var nextTaskParamList = [];
    if (budgetList) {
      budgetList.forEach(function (budget) {
        var obj = nextTaskList.find(function (task) {
          return task.taskDefinitionKey === budget;
        });
        if (obj) {
          obj.category = obj.taskCategory;
          obj.taskDefKey = obj.taskDefinitionKey;
          obj.nextTaskAssigneeParmList = [];
          nextTaskParamList.push(obj);
        }
      });
    }
    return nextTaskParamList;
  },
  /**
   * 判断下一步预演匹配的任务，是否需要弹框选人,
   * 判断规则：
   * 1. currTaskOperate.userChooseMethod == 1 单选(默认), 2: 多选, 3: 全选
   * 2. 只要下一步处理人有多个，都需要弹框选择
   * @param {Array} matchedTaskList
   * @param cb
   * @return {boolean}
   */
  judgeNeedChooseUser: function judgeNeedChooseUser(matchedTaskList, cb) {
    var need = false;
    if (Array.isArray(matchedTaskList)) {
      for (var i = 0, len = matchedTaskList.length; i < len; i++) {
        var taskInfo = matchedTaskList[i];
        if (taskInfo.userChooseMethod !== USER_CHOOSE_METHOD.ALL) {
          if (taskInfo.nextTaskAssigneeParmList && taskInfo.nextTaskAssigneeParmList.length > 1) {
            return cb && cb(true);
          }
        }
      }
    }
    cb && cb(need);
  },
  /**
   * 设置 下一步任务提交参数
   * @param matchNextTaskList
   * @param nextTaskMap 流程选人结果
   * @return {Array}
   */
  setNextTaskParamList: function setNextTaskParamList(matchNextTaskList, nextTaskMap) {
    var nextTaskParamList = [];
    matchNextTaskList.forEach(function (task) {
      var tmpTask = {};
      nextTaskParamListKeys.forEach(function (key) {
        tmpTask[key] = task[key];
      });
      if (nextTaskMap && nextTaskMap[task.taskDefKey]) {
        tmpTask.nextTaskAssigneeParmList = nextTaskMap[task.taskDefKey].nextAssignList;
      }
      nextTaskParamList.push(tmpTask);
    });
    return nextTaskParamList;
  }
};
export default ProcessUtil;