import { EdgeConfig, Graph, IGroup, IShape, Item, ModelConfig, NodeConfig } from "@antv/g6";
import { IG6GraphEvent } from "@antv/g6-core";
export declare type Clazz = 'START' | 'END' | 'NORMAL' | 'PARALLEL_SIGN' | 'SERVICE';
export declare enum WorkFlowNodeEnum {
    START = "START",
    END = "END",
    NORMAL = "NORMAL",
    PARALLEL_SIGN = "PARALLEL_SIGN",
    SERIAL_MULTI_SIGN = "SERIAL_MULTI_SIGN",
    SERVICE = "SERVICE"
}
export interface Events {
    onSelectedItem: (id: string, model: object) => void;
    onBlurItem: () => void;
    onUpdateToolbar: (commands: string[]) => void;
    onValidateError: (item: Item, info: {
        message: string;
        translate: string;
    }) => void;
    onItemClick?: (id: string, model: object) => void;
}
export interface Options {
    mode: 'default' | 'view' | 'edit';
    container: HTMLDivElement;
    plugins?: Array<any>;
    width: number;
    height: number;
    zoom?: boolean;
    tooltip: {
        formatText: (model: any) => string;
        shouldBegin: (e?: IG6GraphEvent) => boolean;
    };
    wheelEnable?: boolean;
    flowTypes?: any[];
    canvasWrapper?: HTMLDivElement;
    itemWrapper?: HTMLDivElement;
    itemSelector?: string;
    toolbar?: HTMLDivElement;
    events: Events;
    defaultData?: GraphData;
    setDynamicData?: (node: any, dynamicData: any) => void;
}
export interface RoleInfo {
    roleId?: string;
    roleName?: string;
    organizationIds?: string;
    organizationNames?: string;
}
export interface INodeGroup extends IGroup {
    anchorShapes: [IShape, IShape][];
    resizeShapes: IShape[];
    showAnchor(group: INodeGroup, show?: boolean | string): void;
    showHotpot(group: INodeGroup, show?: boolean | string): void;
    showResize(group: INodeGroup, show?: boolean | string): void;
    hideAnchor(): void;
    shapeMap: {
        [key: string]: IShape;
    };
    minSize?: [number, number];
}
export interface BehaviorThis {
    graph: Graph;
}
export interface FlowNode extends ModelConfig {
    id: string;
    clazz: Clazz;
    haveOpinionText?: boolean;
    necOpinionText?: boolean;
    canEditPage?: boolean;
    haveAddSign?: boolean;
    haveDelegate?: boolean;
    passRead?: boolean;
    haveCounterSign?: boolean;
    canReadAtt?: boolean;
    canDownloadAtt?: boolean;
    canUploadAtt?: boolean;
    canUpdateMyAtt?: boolean;
    canUpdateOtherAtt?: boolean;
    canDeleteMyAtt?: boolean;
    canDeleteOtherAtt?: boolean;
    canAppEdit?: boolean;
    isProcFirst?: boolean;
    taskTimeout?: string;
    serviceTaskValue?: string;
    compCondition?: string;
    userChooseType?: string;
    userChooseClass?: string;
    userChooseMethod?: string;
    showHideFields?: string;
    editFields?: string;
    groupId?: number;
    parentGroupId?: number;
    mulInstKey?: string;
    nodeKey?: string;
    roleIds?: RoleInfo[];
    x?: number;
    y?: number;
    size?: [number, number];
    defCategory?: Clazz;
    defKey?: string;
    defName?: string;
    nodeIdIndex?: number;
}
export interface FlowEdge extends EdgeConfig {
    sourceTaskDefKey?: string;
    tarTaskDefKey?: string;
    flowName?: string;
    flowType?: string;
    canSendMsg?: boolean;
    canSendWeChat?: boolean;
    canSendEMail?: boolean;
    canAuto?: boolean;
    abnormalProc?: boolean;
    expression?: string;
    remark?: string;
    isServiceOutLine?: boolean;
    stroke?: string;
    [key: string]: any;
}
export interface GraphData {
    edges: EdgeConfig[];
    nodes: NodeConfig[];
    [key: string]: any;
}
